searchData={"items":[{"type":"module","title":"Lens2","doc":"`Use` this module for convenience.\n\nA module that does that can...\n\n1. ... access all the lens-making functions from the [Lens 1](https://hexdocs.pm/lens/readme.html) package\n   under the same names: `Lens.key/1`, for example. See\n   `Lens2.Lenses` for the complete list.\n\n2. ... make lenses for `MapSet` containers with `Lens2.MapSet` and\n   lenses for `BiMap` with `Lens2.BiMap`. Those are aliases for\n   `Lens2.Lenses.MapSet` and `Lens2.Lenses.BiMap`.\n\n3. ... traverse containers with lens-using functions like\n   `Deeply.update`. (See `Lens2.Deeply`.)\n\n3. ... define its own lens makers with `def_raw_maker` and `defmaker`. (See\n   `Lens2.Makers`.)\n\n\nIt also defines the types used in specs.","ref":"Lens2.html"},{"type":"macro","title":"Lens2.__using__/1","doc":"Provide standard imports and aliases.\n\nSee the top of this page for more.","ref":"Lens2.html#__using__/1"},{"type":"type","title":"Lens2.container/0","doc":"A nested data structure described by a `lens`.","ref":"Lens2.html#t:container/0"},{"type":"type","title":"Lens2.lens/0","doc":"Conceptually, a set of pointers into specified locations in a `container`.","ref":"Lens2.html#t:lens/0"},{"type":"type","title":"Lens2.updated_value/0","doc":"A `value`, but named differently so it's clear it's been updated by the application of some function.","ref":"Lens2.html#t:updated_value/0"},{"type":"type","title":"Lens2.value/0","doc":"The value at a place pointed to by a `lens`.","ref":"Lens2.html#t:value/0"},{"type":"module","title":"Lens2.Deeply","doc":"Operations that work with lenses. The API is close to the familiar `get`, `put`, `update` one.\n\nNote, though, these differences:\n\n1. Most APIs with `get` (like `Map` and `Keyword`) return a\n   single value from a container. Lenses point at zero or more\n   places within a container, so the natural \"read\" operation is to\n   return a collection (specifically, a `List`). Naming that\n   operation `get` invites mistakes, so the fundamental \"read\"\n   operation is `Lens2.Deeply.get_all/2`. In the case where you know there's\n   a single value being returned, you can use `Lens2.Deeply.get_only/2` to avoid having\n   to pick a value out of a singleton list.\n\n2. In those Elixir core APIs, it's the operations that decides what to do about\n   missing keys and the like. Consider `Map.update/4` (use a default)\n   and `Map.update!/3` (raise an error). In this lens package, it's the\n   *lens* that decides. See, for example, `Lens2.Lenses.Keyed.key/1`,\n   `Lens2.Lenses.Keyed.key?/1`, and `Lens2.Lenses.Keyed.key!/1`. Or see [this page in the tutorial](tutorial04-missing.html).\n\n3. When you have structs, you can add lenses to the module interface so that you can\n   write code like:\n\n        Deeply.put(%Container{...}, Container.place(...), ...)\n\n   In the reasonably-common case where the module's lens function takes no argument, you\n   can just use the bare atom name:\n\n        Deeply.put(%Container{...}, :place, ...)\n\n   See [this page](tutorial06-information-hiding.html) for more.\n\nThe functions in this module have simple implementations because lenses are\ncompatible with the `Access` behaviour. For example,\n`Lens2.Deeply.put/3` is just a wrapper around a call to\n`put_in/3`:\n\n     def put(container, lens, value),\n         do: put_in(container, [lens], value)\n\nYou can use the `Kernel` functions if you prefer. Don't forget the bracket!\n\nFor convenience and backwards compatibility, this package also provides the [Lens 1](https://hexdocs.pm/lens/readme.html) functions `to_list/2`, and `one!/2`.","ref":"Lens2.Deeply.html"},{"type":"function","title":"Lens2.Deeply.each/3","doc":"Call a function (for side effects) on each pointed-at value.\n\n    iex>  container = %{a: 1,  b: 2, c: 3}\n    iex>  lens = Lens.keys([:a, :c])\n    iex>  Deeply.each(container, lens, fn value ->\n    ...>    send(self(), \"sending #{value}\")\n    ...>  end)\n    iex>  assert_receive(\"sending 1\")\n    iex>  assert_receive(\"sending 3\")\n\nAny return value from the `fun` is ignored.","ref":"Lens2.Deeply.html#each/3"},{"type":"function","title":"Lens2.Deeply.get_all/2","doc":"Returns a list of the values that the lens points at.\n\n    iex>  lens = Lens.map_values\n    iex>  %{a: 1, b: 2, c: 3} |> Deeply.get_all(lens) |> Enum.sort\n    [          1,    2,    3]\n\n`get_all` produces its result with `get_in/2`.","ref":"Lens2.Deeply.html#get_all/2"},{"type":"function","title":"Lens2.Deeply.get_and_update/3","doc":"Retrieve all pointed-at values and, at the same time, update them.\n\nThe return value is a tuple with two elements:\n1. A list (as with `get_all/2`) of the values *before* the update.\n2. The entire container, updated as with `update/3`.\n\n       iex>  lens = Lens.keys([:a, :c])\n       ...>  fun = fn value ->\n       ...>    {value, value * 1000}\n       ...>  end\n       iex>  %{a:    1,  b: 2,  c:    3} |> Deeply.get_and_update(lens, fun)\n       {     [       1,               3],\n             %{a: 1000,  b: 2,  c: 3000}\n       }\n\n`get_and_update` produces its result with `get_and_update_in/3`.","ref":"Lens2.Deeply.html#get_and_update/3"},{"type":"function","title":"Lens2.Deeply.get_only/2","doc":"Return the single value the lens points at.\n\nThis calls `get_all/2` and unwraps the single value in the resulting list.\nIf that list has a different number of elements, a `MatchError` will be raised.\n\n    iex>  lens = Lens.key(:a)\n    iex>  %{a: 1, b: 2, c: 3} |> Deeply.get_only(lens)\n    1","ref":"Lens2.Deeply.html#get_only/2"},{"type":"function","title":"Lens2.Deeply.one!/2","doc":"Return the single value the lens points at.\n\nThis is the name the `get_only/2` function had in [Lens 1](https://hexdocs.pm/lens/readme.html). I prefer `get_only/2`,\nbut if you prefer this name, more power to you.\n\nThis calls `get_all/2` and unwraps the single value in the resulting list.\nIf that list has a different number of elements, a `MatchError` will be raised.\n\n    iex>  lens = Lens.key(:a)\n    iex>  %{a: 1, b: 2, c: 3} |> Deeply.one!(lens)\n    1","ref":"Lens2.Deeply.html#one!/2"},{"type":"function","title":"Lens2.Deeply.put/3","doc":"Put a specific value into all the places the lens points at.\n\n    iex>  lens = Lens.keys([:a, :c])\n    iex>  %{a:    1,  b: 2,  c:    3} |> Deeply.put(lens, :NEW)\n    %{      a: :NEW,  b: 2,  c: :NEW}\n\n`put` produces its result with `put_in/3`.","ref":"Lens2.Deeply.html#put/3"},{"type":"function","title":"Lens2.Deeply.to_list/2","doc":"Returns a list of the values that the lens points at.\n\nThis is the name the `get_all/2` function had in [Lens 1](https://hexdocs.pm/lens/readme.html). I prefer `get_all/2`,\nnote least because if I forget and use `Deeply.get`, I get a helpful error message:\n\n    Lens2.Deeply.get/2 is undefined or private. Did you mean:\n\n    * get_all/2\n    * get_only/2\n\nHowever, `to_list` also has its a strong legacy in the Elixir\ntradition, so you choose.\n\n    iex>  lens = Lens.map_values\n    iex>  %{a: 1, b: 2, c: 3} |> Deeply.to_list(lens) |> Enum.sort\n    [          1,    2,    3]\n\n`get_all` produces its result with `get_in/2`.","ref":"Lens2.Deeply.html#to_list/2"},{"type":"function","title":"Lens2.Deeply.update/3","doc":"Apply the given function to every value pointed at by the lens,\nreturning an updated container.\n\n    iex>  lens = Lens.keys([:a, :c])\n    iex>  updater = & &1 * 1000\n    iex>  %{a:    1,  b: 2,  c:    3} |> Deeply.update(lens, updater)\n    %{      a: 1000,  b: 2,  c: 3000}\n\nThis corresponds to `map` in the original\n[`Lens`]((https://hexdocs.pm/lens/readme.html)) package. The name\nhas been changed for consistency with `Access`.\n\n`update` produces its result with `update_in/3`.","ref":"Lens2.Deeply.html#update/3"},{"type":"type","title":"Lens2.Deeply.lens_or_atom/0","doc":"","ref":"Lens2.Deeply.html#t:lens_or_atom/0"},{"type":"module","title":"Lens2.Makers","doc":"Two ways of defining named lens-making functions that improve on `def`.\n\n*Lens makers* are named functions that, when called, return a\n lens. Lens makers are created in three ways:\n\n1. Using plain old `def`:\n\n       def leaf(key), do: Lens.key(:down) |> Lens.key(key)\n\n   This is fine, but you *cannot* use `leaf` itself in a pipeline. That is, this\n   will fail to compile:\n\n       Lens.key(:upper) |> leaf(:bottom)\n       error: undefined function leaf/2\n\n1. Because of that, it's better to use `defmaker/2` (alternately, `deflens/2`).\n   This creates a lens maker from one or more other lens makers, most often\n   by piping them into each other. For example, a lens maker for two-level maps\n   could be defined like this:\n\n       defmaker leaf(key), do: Lens.key(:down) |> Lens.key(key)\n       # Works fine:\n       Lens.key(:upper) |> leaf(:bottom)\n\n1. `def_raw_maker/2` (alternately, `deflens_raw/2`, is used for cases where\n   a lens maker can't be made from other lens makers. For example, the definition of\n   ``Lens2.Lenses.Keyed.key!/1`` looks like this:\n\n       @spec key!(any) :: Lens2.lens\n       def_raw_maker key!(key) do\n         fn composed, descender ->\n           {gotten, updated} = descender.(DefOps.fetch!(composed, key))\n           {[gotten], DefOps.put(composed, key, updated)}\n         end\n       end","ref":"Lens2.Makers.html"},{"type":"macro","title":"Lens2.Makers.__using__/1","doc":"Import the maker macros.","ref":"Lens2.Makers.html#__using__/1"},{"type":"macro","title":"Lens2.Makers.def_raw_maker/2","doc":"Write a lens maker without using other lens makers.\n\nThe body of the function must return a two-argument function that takes a container\nand a \"descender\" function:\n\n    @spec key!(any) :: Lens2.lens\n    def_raw_maker key!(key) do\n      fn container, descender ->\n        ...\n      end\n    end\n\nThe job of the returned function is to take the container, take a\nvalue within it, and pass it to the descender:\n\n    def_raw_maker key(key) do\n      fn container, descender ->\n        ... = descender.(DefOps.fetch!(container, key))\n        ...\n      end\n    end\n\n(`Defops.fetch!` calls `Map.fetch!/2` in the case of a map or\n`Access.fetch!/2` otherwise. I presume the map version of `fetch!`\nis more efficient than the `Access` version.)\n\nThe result is a tuple:\n\n* If the original container was being operated on by\n  `Lens2.Deeply.get_and_update/3` or `get_and_update_in/3`, the first\n  element contains the \"get\" and the second is the \"update\".\n\n* If the operation was a put or an update, the same is done – I\n  guess you've got the value right there, so you might as well pass it along. The\n  original operation will discards it.\n\n* If the operation is a get, the gotten value will be both values of the tuple.\n\nYour code doesn't have to worry about that: it can simply assume\n`get_and_update` is being used.\n\n      fn container, descender ->\n        {gotten, updated} = descender.(...)\n        ...\n      end\n    end\n\nYour code has to bundle up the `gotten` and `updated` functions into\na tuple of its own and return that:\n\n     fn container, descender ->\n       {gotten, updated} = descender.(DefOps.fetch!(container, key))\n       {[gotten], DefOps.put(container, key, updated)}\n     end\n\nThe \"updated\" case involves putting the new value in place of the\none that was extracted \"on the way down\". The \"gotten\" case is more\nsubtle **and I haven't figured out how to explain it. This paragraph\nshould link to a tutorial I haven't written yet.**","ref":"Lens2.Makers.html#def_raw_maker/2"},{"type":"macro","title":"Lens2.Makers.deflens/2","doc":"Alternate spelling of `defmaker/2`.\n\nThis is the equivalent macro from [Lens 1](https://hexdocs.pm/lens/readme.html). You won't break my heart\nif you prefer it to `defmaker/1`, but I had early troubles confusing\na lens and a lens *maker*. I suspect this name will help other novices avoid my\nmistakes.\n\nI suppose I should deprecate the `Lens` alias (`Lens.key?(:a)`, etc.), but:\n\n> Do I contradict myself?   \n> Very well then I contradict myself,   \n> (I am large, I contain multitudes.)   \n>   \n>    – Walt Whitman, Leaves of Grass","ref":"Lens2.Makers.html#deflens/2"},{"type":"macro","title":"Lens2.Makers.deflens_raw/2","doc":"Alternate spelling of `def_raw_maker/2`.\n\nThis is the equivalent macro from [Lens 1](https://hexdocs.pm/lens/readme.html).","ref":"Lens2.Makers.html#deflens_raw/2"},{"type":"macro","title":"Lens2.Makers.defmaker/2","doc":"Create a lens maker from one or more other lens makers.\n\nMost often, this is used to name a pipeline of lenses (and make the\nresulting function suitable for including in other pipelines):\n\n     defmaker leaf(key), do: Lens.key(:down) |> Lens.key(key)\n\nThe body after the `do:` can be arbitrary code:\n\n     defmaker keys(keys) do\n       keys |> Enum.map(&key/1) |> Combine.multiple\n     end\n\nThe only requirement is that the last thing the code does must be to\ncall a lens maker.  (Or, I suppose, produce a lens in some other\nway, but you'd probably use `def_raw_maker/2` for that.)","ref":"Lens2.Makers.html#defmaker/2"},{"type":"task","title":"mix has.smart.put","doc":"Is the data type smart about \"put\" operations that produce identical structures?\n\nA naive implementation of `Map` would be such that:\n\n    iex> Map.get(map, :key)\n    :some_value\n    iex> Map.put(map, :key, :some_value)\n\n... would allocate a completely new map. Maps and structs are, in fact, smarter than\nthat. `put` just returns the original map.\n\nThis matters for the current implementation of lenses because\n`Deeply.get_all` will do such equality-preserving puts (and then\nthrow the result away). Naive implementations of a container data\ntype will do extra work that `get_in` avoids.\n\nIf you're implementing lenses for a new data structure, you nmight want to know\nwhether it's smart or naive.\n\n    $ mix has.smart.put\n    Is the data structure smart enough to not create an identical copy?\n    List: false\n    Map: true\n    Struct: true\n    MapSet: true","ref":"Mix.Tasks.Has.Smart.Put.html"},{"type":"task","title":"mix timings.keyed","doc":"Crude timings that compare Access and Lens operations on a struct+map container.\n\nThe container has this structure:\n\n     .Network{\n       cluster_count: 10,\n       names_by_index: %{\n         1 => :name1,\n         2 => :name2,\n         3 => :name3,\n         ...\n       },\n       clusters_by_name: %{\n         name1: %Mix.Tasks.Timings.Keyed.Cluster{\n           router: %{c: 0, a: 0, d: 0, b: 0, e: 0},\n           fun: &Kernel.inspect/1,\n           string: \":name1\",\n           atom1: :name1\n         },\n         name2: %Mix.Tasks.Timings.Keyed.Cluster{\n           router: %{c: 0, a: 0, d: 0, b: 0, e: 0},\n           fun: &Kernel.inspect/1,\n           string: \":name2\",\n           atom1: :name2\n         },\n         ...\n       }\n     }\n\nGet operations (get_in, Deeply.get_all) extract values like\n\n     network.clusters_by_name[:name2].router.[:c]\n\nUpdate operations increment those values.\n\nCall with a list of the number of clusters:\n\n      % mix timings.keyed 10 100 1000\n\nEach number creates a scaled network and runs the operation 40_000_000 times.\n\nI wrote this because I realized that `Deeply.get_all` will create a\ncopy of the original structure as it \"retreats\" from finding the\ngotten values. Fortunately, for maps and structs,\n`Map.put(container, key, new_value)` actually returns the original\n`container` if the new value is the same as the existing value, so\nthat's not so bad. But it's interesting to compare Access to Lens anyway.","ref":"Mix.Tasks.Timings.Keyed.html"},{"type":"task","title":"mix timings.list","doc":"Crude timings that compare Access and Lens operations on a struct+map container.\n\nThe container has this structure:\n\n     .Network{\n       cluster_count: 10,\n       names_by_index: %{\n         1 => :name1,\n         2 => :name2,\n         3 => :name3,\n         ...\n       },\n       clusters_by_name: %{\n         name1: %Mix.Tasks.Timings.Keyed.Cluster{\n           router: %{c: 0, a: 0, d: 0, b: 0, e: 0},\n           fun: &Kernel.inspect/1,\n           string: \":name1\",\n           atom1: :name1\n         },\n         name2: %Mix.Tasks.Timings.Keyed.Cluster{\n           router: %{c: 0, a: 0, d: 0, b: 0, e: 0},\n           fun: &Kernel.inspect/1,\n           string: \":name2\",\n           atom1: :name2\n         },\n         ...\n       }\n     }\n\nGet operations (get_in, Deeply.get_all) extract values like\n\n     network.clusters_by_name[:name2].router.[:c]\n\nUpdate operations increment those values.\n\nCall with a list of the number of clusters:\n\n      % mix timings.keyed 10 100 1000\n\nEach number creates a scaled network and runs the operation 40_000_000 times.\n\nI wrote this because I realized that `Deeply.get_all` will create a\ncopy of the original structure as it \"retreats\" from finding the\ngotten values. Fortunately, for maps and structs,\n`Map.put(container, key, new_value)` actually returns the original\n`container` if the new value is the same as the existing value, so\nthat's not so bad. But it's interesting to compare Access to Lens anyway.","ref":"Mix.Tasks.Timings.List.html"},{"type":"module","title":"Lens2.Lenses","doc":"Aggregates all the lenses in\n`Lens2.Lenses.Combine`,\n`Lens2.Lenses.Enum`,\n`Lens2.Lenses.Filter`,\n`Lens2.Lenses.Indexed`, and\n`Lens2.Lenses.Keyed`.\n\nTraditionally, this module is aliased to `Lens`, so that the makers\nhave the same name as in the [Lens 1](https://hexdocs.pm/lens/readme.html) package. See the `Lens2` module.","ref":"Lens2.Lenses.html"},{"type":"function","title":"Lens2.Lenses.all/0","doc":"","ref":"Lens2.Lenses.html#all/0"},{"type":"function","title":"Lens2.Lenses.and_repeatedly/1","doc":"","ref":"Lens2.Lenses.html#and_repeatedly/1"},{"type":"function","title":"Lens2.Lenses.at/1","doc":"","ref":"Lens2.Lenses.html#at/1"},{"type":"function","title":"Lens2.Lenses.back/0","doc":"","ref":"Lens2.Lenses.html#back/0"},{"type":"function","title":"Lens2.Lenses.before/1","doc":"","ref":"Lens2.Lenses.html#before/1"},{"type":"function","title":"Lens2.Lenses.behind/1","doc":"","ref":"Lens2.Lenses.html#behind/1"},{"type":"function","title":"Lens2.Lenses.both/2","doc":"","ref":"Lens2.Lenses.html#both/2"},{"type":"function","title":"Lens2.Lenses.const/1","doc":"","ref":"Lens2.Lenses.html#const/1"},{"type":"function","title":"Lens2.Lenses.context/2","doc":"","ref":"Lens2.Lenses.html#context/2"},{"type":"function","title":"Lens2.Lenses.either/2","doc":"","ref":"Lens2.Lenses.html#either/2"},{"type":"function","title":"Lens2.Lenses.empty/0","doc":"","ref":"Lens2.Lenses.html#empty/0"},{"type":"function","title":"Lens2.Lenses.filter/1","doc":"","ref":"Lens2.Lenses.html#filter/1"},{"type":"function","title":"Lens2.Lenses.filter/2","doc":"","ref":"Lens2.Lenses.html#filter/2"},{"type":"function","title":"Lens2.Lenses.front/0","doc":"","ref":"Lens2.Lenses.html#front/0"},{"type":"function","title":"Lens2.Lenses.index/1","doc":"","ref":"Lens2.Lenses.html#index/1"},{"type":"function","title":"Lens2.Lenses.indices/1","doc":"","ref":"Lens2.Lenses.html#indices/1"},{"type":"function","title":"Lens2.Lenses.into/2","doc":"","ref":"Lens2.Lenses.html#into/2"},{"type":"function","title":"Lens2.Lenses.key/1","doc":"","ref":"Lens2.Lenses.html#key/1"},{"type":"function","title":"Lens2.Lenses.key!/1","doc":"","ref":"Lens2.Lenses.html#key!/1"},{"type":"function","title":"Lens2.Lenses.key?/1","doc":"","ref":"Lens2.Lenses.html#key?/1"},{"type":"function","title":"Lens2.Lenses.key_path!/1","doc":"","ref":"Lens2.Lenses.html#key_path!/1"},{"type":"function","title":"Lens2.Lenses.keys/1","doc":"","ref":"Lens2.Lenses.html#keys/1"},{"type":"function","title":"Lens2.Lenses.keys!/1","doc":"","ref":"Lens2.Lenses.html#keys!/1"},{"type":"function","title":"Lens2.Lenses.keys?/1","doc":"","ref":"Lens2.Lenses.html#keys?/1"},{"type":"function","title":"Lens2.Lenses.map_keys/0","doc":"","ref":"Lens2.Lenses.html#map_keys/0"},{"type":"function","title":"Lens2.Lenses.map_values/0","doc":"","ref":"Lens2.Lenses.html#map_values/0"},{"type":"function","title":"Lens2.Lenses.match/1","doc":"","ref":"Lens2.Lenses.html#match/1"},{"type":"function","title":"Lens2.Lenses.multiple/1","doc":"","ref":"Lens2.Lenses.html#multiple/1"},{"type":"function","title":"Lens2.Lenses.recur/1","doc":"","ref":"Lens2.Lenses.html#recur/1"},{"type":"function","title":"Lens2.Lenses.recur_root/1","doc":"","ref":"Lens2.Lenses.html#recur_root/1"},{"type":"function","title":"Lens2.Lenses.reject/2","doc":"","ref":"Lens2.Lenses.html#reject/2"},{"type":"function","title":"Lens2.Lenses.repeatedly/1","doc":"","ref":"Lens2.Lenses.html#repeatedly/1"},{"type":"function","title":"Lens2.Lenses.root/0","doc":"","ref":"Lens2.Lenses.html#root/0"},{"type":"function","title":"Lens2.Lenses.seq/2","doc":"","ref":"Lens2.Lenses.html#seq/2"},{"type":"function","title":"Lens2.Lenses.seq_both/2","doc":"","ref":"Lens2.Lenses.html#seq_both/2"},{"type":"function","title":"Lens2.Lenses.update_into/2","doc":"","ref":"Lens2.Lenses.html#update_into/2"},{"type":"module","title":"Lens2.Lenses.BiMap","doc":"Lens makers for the [`BiMap`](https://hexdocs.pm/bimap/readme.html)\nbidirectional map [package](https://hex.pm/packages/bimap).\n\nA `BiMap` allows a \"reverse lookup\": using a value to find the corresponding key.\n\n    iex>  bimap = BiMap.new(%{1 => \"1111\", 2 => \"2222\"})\n    iex>  BiMap.get(bimap, 1)\n    \"1111\"\n    iex>  BiMap.get_key(bimap, \"1111\")\n    1\n\nA `BiMap` differs importantly from a map in that it requires\n*values*, not just keys, to be unique. This can cause some confusion\nas BiMap's `put` operation can cause existing bindings to disappear.\n\n     iex>  bimap = BiMap.new(a: 5, b: 6)\n     iex>  BiMap.put(bimap, :b, 5)\n     BiMap.new(b: 5)   # where did `:a` go?\n\nI've found the problem gets worse with lenses (because you may be\nputting multiple places in a single call, and the place you want\nto change may be nested below the BiMap, making it harder to\nnotice the constraint is relevant.) You might prefer\n[BiMultiMap](https://hexdocs.pm/bimap/BiMultiMap.html) (which comes with BiMap).\n\nNote that `BiMap` does not implement `Access`. However, lenses\ncreated with these functions do, so they can be used with `get_in/2`\nand friends.\n\nThis module follows the convention of using names like `key` to mean\nusing a key to obtain a value. Names like `to_key` go in the reverse\ndirection: from value to key. Using `value(1)` to refer to the key\n`:a` would be just too weird.","ref":"Lens2.Lenses.BiMap.html"},{"type":"function","title":"Lens2.Lenses.BiMap.all_keys/0","doc":"Return a lens that points at all keys of a [`BiMap`](https://hexdocs.pm/bimap/readme.html).\n\nThis is useful for descending into complex keys. Consider a\nspatial BiMap that connects `{x, y}` tuples to some values\nrepresenting physical objects at that position.\n\n    iex>  bimap = BiMap.new(%{{0, 0} => \"some data\", {1, 1} => \"other data\"})\n    iex>  Deeply.update(bimap, Lens.BiMap.all_keys,\n    ...>                       fn {x, y} -> {x + 1, y + 1} end)\n    iex>  BiMap.new(%{{1, 1} => \"some data\", {2, 2} => \"other data\"})\n\nNote that all the updates are done before the result BiMap is formed. You needn't fear\nthat updating the `{0, 0}` tuple will wipe out the existing `{1, 1}` tuple.","ref":"Lens2.Lenses.BiMap.html#all_keys/0"},{"type":"function","title":"Lens2.Lenses.BiMap.all_values/0","doc":"Return a lens that points at all values of a [`BiMap`](https://hexdocs.pm/bimap/readme.html).\n\n    iex>  bimap = BiMap.new(%{1 => \"1111\", 2 => \"2222\"})\n    iex>  Deeply.get_all(bimap, Lens.BiMap.all_values) |> Enum.sort\n    [\"1111\", \"2222\"]\n\nNote that using `put` with `all_values` is profoundly useless, as\nonly one key/value pair can be retained – and you can't even predict\nwhich one it will be!\n\n    iex>  bimap = BiMap.new(a: 1, b: 2, c: 3, d: 4, e: 5)\n    iex>  updated = Deeply.put(bimap, Lens.BiMap.all_values, :NEW)\n    iex>  assert [:NEW] == BiMap.values(updated)\n    ...>\n    iex>  [key] = BiMap.keys(updated)\n    iex>  assert key in [:a, :b, :c, :d, :e]\n    ...>  # On my machine, today, it turns out to be `:e`.","ref":"Lens2.Lenses.BiMap.html#all_values/0"},{"type":"function","title":"Lens2.Lenses.BiMap.key/1","doc":"Return a lens that points at the value of a single [`BiMap`](https://hexdocs.pm/bimap/readme.html) key.\n\nAs with `Lens2.Lenses.Keyed.key/1`, a missing key is represented with `nil`:\n\n    iex>  bimap = BiMap.new(a: 1, b: 2)\n    ...>  lens = [:a, :MISSING] |> Enum.map(&Lens.BiMap.key/1) |> Lens.multiple\n    iex>  Deeply.get_all(bimap, lens) |> Enum.sort\n    [1, nil]\n\nThis lens can introduce a missing key-value pair, unlike `key?/1`.\n\n    iex> Deeply.put(BiMap.new, Lens.BiMap.key(:missing), :NEW)\n    BiMap.new(%{missing: :NEW})","ref":"Lens2.Lenses.BiMap.html#key/1"},{"type":"function","title":"Lens2.Lenses.BiMap.key!/1","doc":"Like `key/1` and `key?/1` except that the lens will raise an error for a missing key.\n\nThis works the same as `Lens2.Lenses.Keyed.key!/1`.\n\n    iex>  bimap = BiMap.new(a: 1)\n    iex>  Deeply.put(bimap, Lens.BiMap.key!(:a), :NEW)\n    BiMap.new(a: :NEW)\n    iex>  Deeply.put(bimap, Lens.BiMap.key!(:missing), :NEW)\n    ** (ArgumentError) key :missing not found in: BiMap.new([a: 1])","ref":"Lens2.Lenses.BiMap.html#key!/1"},{"type":"function","title":"Lens2.Lenses.BiMap.key?/1","doc":"Return a lens that points at the value of a single\n[`BiMap`](https://hexdocs.pm/bimap/readme.html) key, ignoring\nmissing keys.\n\nThis works the same as `Lens2.Lenses.Keyed.key?/1`.\n\n    iex>  bimap = BiMap.new(a: 1, b: 2)\n    ...>  lens = [:a, :MISSING] |> Enum.map(&Lens.BiMap.key?/1) |> Lens.multiple\n    iex>  Deeply.get_all(bimap, lens)\n    [1]\n\nUnlike `key/1`, the returned lens cannot be used to create a missing key:\n\n    iex> Deeply.put(BiMap.new, Lens.BiMap.key?(:missing), :NEW)\n    BiMap.new","ref":"Lens2.Lenses.BiMap.html#key?/1"},{"type":"function","title":"Lens2.Lenses.BiMap.keys/1","doc":"Like `key/1` but takes a list of keys.\n\nThe value of a missing key is treated as `nil`.\n\n    iex>  bimap = BiMap.new(a: 2)\n    iex>  lens = Lens.BiMap.keys([:a, :missing])\n    iex>  updater = fn\n    ...>    nil -> \"newly added\"\n    ...>    x   -> x * 1111\n    ...>  end\n    iex>  Deeply.update(bimap, lens, updater)\n    BiMap.new(a: 2222, missing: \"newly added\")","ref":"Lens2.Lenses.BiMap.html#keys/1"},{"type":"function","title":"Lens2.Lenses.BiMap.keys!/1","doc":"Like `key!/1` but takes a list of keys.\n\nAny missing key raises an error.\n\n    iex>  bimap = BiMap.new(a: 2)\n    iex>  lens = Lens.BiMap.keys!([:a, :missing])\n    iex>  Deeply.get_only(bimap, lens)\n    ** (ArgumentError) key :missing not found in: BiMap.new([a: 2])","ref":"Lens2.Lenses.BiMap.html#keys!/1"},{"type":"function","title":"Lens2.Lenses.BiMap.keys?/1","doc":"Like `key?/1` but takes a list of keys.\n\nMissing keys are ignored.\n\n    iex>  bimap = BiMap.new(a: 2)\n    iex>  lens = Lens.BiMap.keys?([:a, :missing])\n    iex>  Deeply.update(bimap, lens, & &1*1111)\n    BiMap.new(a: 2222)\n    iex>  Deeply.get_only(bimap, lens)\n    2","ref":"Lens2.Lenses.BiMap.html#keys?/1"},{"type":"function","title":"Lens2.Lenses.BiMap.to_key!/1","doc":"Return a lens that points at the key associated with a given value. Raises an\nerror if there is no such value.\n\n    iex>  bimap = BiMap.new(a: 1)\n    iex>  Deeply.get_all(bimap, Lens.BiMap.to_key!(1))\n    [:a]\n    iex>  Deeply.get_all(bimap, Lens.BiMap.to_key!(11111))\n    ** (ArgumentError) value 11111 not found in: BiMap.new([a: 1])","ref":"Lens2.Lenses.BiMap.html#to_key!/1"},{"type":"function","title":"Lens2.Lenses.BiMap.to_key?/1","doc":"Return a lens that points at the key associated with a given value,\nprovided there is any such value.\n\n    iex>  bimap = BiMap.new(a: 1)\n    iex>  Deeply.get_all(bimap, Lens.BiMap.to_key?(1))\n    [:a]\n    iex>  Deeply.get_all(bimap, Lens.BiMap.to_key?(11111))\n    []\n\nYou can create a new key-value pair with `key/1`.\n`Lens2.Deeply.put/3` will not work with this lens:\n\n    iex> Deeply.put(BiMap.new, Lens.BiMap.to_key?(\"value\"), :new_key)\n    BiMap.new","ref":"Lens2.Lenses.BiMap.html#to_key?/1"},{"type":"function","title":"Lens2.Lenses.BiMap.to_keys!/1","doc":"Like `to_key!/1` but takes a list of values.\n\nAny missing value raises an error.\n\n    iex>  bimap = BiMap.new(a: 2)\n    iex>  lens = Lens.BiMap.to_keys!([2, \"missing value\"])\n    iex>  Deeply.get_all(bimap, lens)\n    ** (ArgumentError) value \"missing value\" not found in: BiMap.new([a: 2])","ref":"Lens2.Lenses.BiMap.html#to_keys!/1"},{"type":"function","title":"Lens2.Lenses.BiMap.to_keys?/1","doc":"Like `to_key?/1` but takes a list of values.\n\nMissing keys are ignored.\n\n    iex>  bimap = BiMap.new(%{1 => \"value\"})\n    iex>  lens = Lens.BiMap.to_keys?([\"value\", \"some missing value\"])\n    iex>  Deeply.update(bimap, lens, & &1*1111)\n    BiMap.new(%{1111 => \"value\"})\n    iex>  Deeply.get_only(bimap, lens)\n    1","ref":"Lens2.Lenses.BiMap.html#to_keys?/1"},{"type":"module","title":"Lens2.Lenses.Combine","doc":"Lenses that combine lenses to get new lenses.","ref":"Lens2.Lenses.Combine.html"},{"type":"function","title":"Lens2.Lenses.Combine.and_repeatedly/1","doc":"Apply a `descender` lens repeatedly to augment the current set of\npointers with all matching pointers below them, descending \"all the\nway into\" the container.\n\nSee `repeatedly/1` for details and a more extensive example. The\nonly difference is that the lens produced by this function will\nretain the original pointers. Suppose you have this structure:\n\n    iex>  nested = %{value: 1,\n    ...>             below: %{value: 2,\n    ...>                      below: %{value: 3}}}\n    iex>  values = Lens.and_repeatedly(Lens.key?(:below)) |> Lens.key(:value)\n    iex>  Deeply.get_all(nested, values) |> Enum.sort\n    [1, 2, 3]\n\n Had `repeatedly/1` been used, the `1` value would not be included in the result.","ref":"Lens2.Lenses.Combine.html#and_repeatedly/1"},{"type":"function","title":"Lens2.Lenses.Combine.both/2","doc":"Convert one pointer into two pointers (usually into the next level down).\n\nSuppose you want a lens that will point at all the elements of an\nenumeration that are divisible by either two or three. The lenses to\npoint at the two types of values can be made with\n`Lens2.Lenses.Filter.filter/2`, then combined with `both/2`:\n\n    iex> by_2 = Lens.filter(& rem(&1, 2) == 0)\n    iex> by_3 = Lens.filter(& rem(&1, 3) == 0)\n    iex> list = [0, 1, 2, 3, 4, 5, 6]\n    iex> lens = Lens.all |> Lens.both(by_2, by_3)\n    iex> Deeply.get_all(list, lens)\n    [0, 0, 2, 3, 4, 6, 6]\n\nYou might be surprised by the duplications in the result (I was!),\nbut that shows that the two lenses are independent. `by_2` produces\npointers to `0`, `2`, `4`, and `6`. `by_3` produces pointers to `0`,\n`3`, and `6`. `both/2`, when used by `Deeply.get_all` just gathers\nthe values at all the pointer, not caring that sometimes two point\nat the same thing.\n\nThe possibility of duplications has consequences for `Deeply.update`. For example,\nconsider this:\n\n    Deeply.update(list, lens, & &1 * 1111)\n\nHere's the result:\n\n    [0, 1, 2222, 3333, 4444, 5, 7405926]\n\nThe original `6` matched both filters, so it was multiplied\ntwice. (So was the `0`, though you can't tell.)","ref":"Lens2.Lenses.Combine.html#both/2"},{"type":"function","title":"Lens2.Lenses.Combine.const/1","doc":"Returns a lens that replaces any incoming pointers with a pointer to the given container.\n\nYou can use this to produce a sort of a default value. For example, recall that\n`Lens2.Lenses.Keyed.key?/1` will return nothing for a missing key:\n\n    iex> Deeply.get_all(%{}, Lens.key?(:a))\n    []\n\n\nThe `either/2` lens will apply its second lens argument if the first\nhas no value, so it can be used to defer to `const`:\n\n    iex> lens = Lens.either(Lens.key?(:a), Lens.const(:DEFAULT))\n    iex> Deeply.get_all(%{a: :GIVEN}, lens)\n    [:GIVEN]\n    iex> Deeply.get_all(%{}, lens)\n    [:DEFAULT]\n\nThe default value can even be used by later lenses:\n\n    iex> lens =\n    ...>   Lens.either(Lens.key?(:a), Lens.const(%{bb: :BB_DEFAULT_VALUE}))\n    ...>   |> Lens.key(:bb)\n    iex> Deeply.get_all(%{a: %{bb: :GIVEN}}, lens)\n    [:GIVEN]\n    iex> Deeply.get_all(%{}, lens)\n    [:BB_DEFAULT_VALUE]\n\nHowever, this defaulting is not as useful as it seems.\n\nFirst, you must take care to use `key?` and not, say,\n`Lens2.Lenses.Keyed.key/1`.  The latter *will* produce a value for a\nmissing key (`nil`), which means `either/2` will not use its second\nargument.\n\nSecond, `Lens2.Deeply.put/3` and `Lens2.Deeply.update/3` work\nbecause a lens pipeline \"backtracks\" along a path to build up the\nnew nested structure. `const` breaks (or, more accurately,\neliminates) that backtracking. For example, in my first use of `const`,\nI did not expect that using `put` on a map with a missing key would \"lose\" the map:\n\n    iex> lens = Lens.either(Lens.key?(:a), Lens.const(:DEFAULT))\n    iex> Deeply.put(%{}, lens, :NEW)\n    :NEW    # *not* %{a: :NEW}\n\nCompare that to using `key/1` or `put_in/1`\n\n    iex> Deeply.put(%{}, Lens.key(:a), :NEW)\n    %{a: :NEW}\n    iex> put_in(%{}, [:a], :NEW)\n    %{a: :NEW}\n\nI think this is all too confusing to justify using `const`.","ref":"Lens2.Lenses.Combine.html#const/1"},{"type":"function","title":"Lens2.Lenses.Combine.context/2","doc":"Record an intermediate container on the way to selected values.\n\nA composed lens will descend through nested containers to reach one\nor more leaf values.  It's occasionally useful to make one or more\nintermediate containers available to the caller of\n`Deeply.get_all` or the function passed to\n`Deeply.update`.\n\n**Getting**\n\nOne use might be to answer the question, \"Where did this leaf value\ncome from?\" Consider the following structure, with a list within a map within a list:\n\n     map_0 = %{a: [0, :target_1, 2], name: \"first\"}\n     map_1 = %{a: [4, :target_5, 6], name: \"second\"}\n     map_list = [map_0, map_1]\n\nWhen fetching the two `:target` atoms, we want to record which map\ncontained them, perhaps to extract the `:name`. We will do that by:\n\n1. Describing a path *from* the map to the target list element:\n\n       into_map = Lens.key(:a) |> Lens.at(1)\n\n2. Marking the start of that path as something worth recording:\n\n       Lens.context(into_map)   # Think of this as taking a snapshot.\n\n3. Combining that with a lens describing the path *to* the map:\n\n       lens = Lens.at(0) |> Lens.context(into_map)\n\nThis lens, applied to `map_list`, will produce two leaf\nvalues. `Deeply.get_all` will return those, but also the snapshots\ntaken on the way. The two are wrapped into a tuple. So:\n\n    [{map_0_snapshot, map_0_target}, {map_1_snapshot, map_1_target}] =\n      Deeply.get_all(map_list, lens)\n\nIn this case, the result will be:\n\n    map_0_snapshot == %{a: [0, :target_1, 2], name: \"first\"}\n    map_0_target   == :target_1\n    map_1_snapshot == %{a: [0, :target_1, 2], name: \"second\"}\n    map_1_target   == :target_5\n\n**Putting**\n\nThe `context` lens has no effect on `Deeply.put`. That is, using the lens above,\nwe could change the targets like this:\n\n    actual = Deeply.put(map_list, lens, :REPLACEMENT)\n\n    actual == [ %{a: [0, :REPLACEMENT, 2], name: \"first\"},\n                %{a: [0, :REPLACEMENT, 2], name: \"second\"}\n\nThat's the same as you'd get with a lens pipeline that leaves out the `context`:\n\n    lens = Lens.at(0) |> Lens.key(:a) |> Lens.at(1)\n\n**Updating**\n\nThe function used with `Deeply.update` will take a tuple argument:\n\n    updater = fn {snapshot, leaf} -> ... end\n\nLet's simplify the previous example to this list-of-maps-of-lists:\n\n    map_list = [\n       %{a: [10, 100, 1000]}\n    ]\n\nWe want to update the `100` element with the sum of the whole `:a`\nlist. For extra fun, we'll add on the current value of the element\n(100), giving an expected result of `10 + 100 + 1000 + 100 = 1210`.\n\nOnce we have the leaf value `100` and the context `{a: [10, 100,\n1000]}`, this updater function would serve:\n\n    updater = fn {snapshot, leaf} ->\n      Enum.sum(snapshot) + leaf\n    end\n\nThen `Deeply.update` will work as hoped:\n\n    actual = Deeply.update(map_list, lens)\n    assert actual == [\n       %{a: [10, 1210, 1000]}\n    ]\n\nNote that `Deeply.update` can still only update leaf values, not the snapshots.\n\n**Nested contexts**\n\nWhat happens with this composed lens?\n\n    Lens.at(0) |> Lens.context(Lens.key(:a) |> Lens.context(Lens.at(1)))\n\nIt produces nested tuples of this form:\n\n    {outer_context, {inner_context, leaf_value}}\n\n... such as:\n\n    {\n      %{a: [0, :target_1, 2], b: 3},\n      {\n        [0, :target_1, 2],\n        :target_1\n      }\n    }\n\nA structure of the same shape and values will be given to a `Deeply.update` function.","ref":"Lens2.Lenses.Combine.html#context/2"},{"type":"function","title":"Lens2.Lenses.Combine.either/2","doc":"Replace pointers with the pointers returned by the first lens. If there are none,\nfollow the second lens.\n\nFor example, to descend through key `:a` or – if there is no such\nkey – then key `:b`:\n\n    iex> lens = Lens.key?(:a) |> Lens.either(Lens.key?(:b))\n    iex> Deeply.get_only(%{a: 1}, lens)\n    1\n    iex> Deeply.get_only(%{b: 2}, lens)\n    2\n\nThe first lens must be a lens that treats missing values differently\nthan nils. Otherwise, the second lens will never be used. Consider changing the\nabove to use `Lens2.Lenses.Keyed.key/1` instead of `Lens2.Lenses.Keyed.key?/1`:\n\n    iex> lens = Lens.key(:a) |> Lens.either(Lens.key?(:b))\n    iex> Deeply.get_only(%{b: 2}, lens)\n    nil\n\nThe `nil` final result is because the first lens produces `nil` for\nthe missing `:a` key.","ref":"Lens2.Lenses.Combine.html#either/2"},{"type":"function","title":"Lens2.Lenses.Combine.empty/0","doc":"Returns a lens ignores all input pointers and produces no pointers.\n\nSuppose you have an `Enumeration` of lenses and you want to combine\nthem into a single lens with `Enum.reduce/3`:\n\n    Enum.reduce(lenses, ???, combining_function)\n\nIf each step of the reduction is going to add a new source of\npointers, the starting point should be a source of *no* pointers,\nwhich is `empty/0`. For example, the implementation of `multiple/1`\ncombines lenses one by one using `both/2`. It starts with `empty/0`:\n\n    Enum.reduce(lenses, empty(), &both/2)","ref":"Lens2.Lenses.Combine.html#empty/0"},{"type":"function","title":"Lens2.Lenses.Combine.match/1","doc":"Descend different \"shapes\" of data differently. Powered by function\nargument pattern matching.\n\n`GenServer` message handlers return tuples to the `GenServer` infrastructure. There are quite a variety, but let's say we only care about these three:\n\n      {:noreply, new_state}\n      {:noreply, new_state, something_else}\n      {:reply, reply, new_state}\n\nWe have an `Enumeration` of these and other such tuples. For our\nthree, we wish to descend into `new_state`. That can be done like\nthis:\n\n     iex>  matcher = fn\n     ...>    {:noreply, _} -> Lens.at(1)\n     ...>    {:noreply, _, _} -> Lens.at(1)\n     ...>    {:reply, _, _} -> Lens.at(2)\n     ...>     _ -> Lens.empty\n     ...>  end\n     iex>  lens = Lens.all |> Lens.match(matcher) |> Lens.key?(:code)\n     iex>  returns = [{:noreply, %{code: 1}},\n     ...>             {:reply, :ok, %{code: 2}},\n     ...>             {:stop, 5, %{code: :ignore}}]\n     iex>  Deeply.get_all(returns, lens)\n     [1, 2]\n\nNote the use of `empty/0` to handle the \"don't care\" case.","ref":"Lens2.Lenses.Combine.html#match/1"},{"type":"function","title":"Lens2.Lenses.Combine.multiple/1","doc":"Use multiple lenses to convert a single pointer into a multitude of\nthem (usually into the next level down).\n\nSuppose you have an enumeration of tuple and want to point to the\nvalues at indices 0, 1, and 3. You can do that like this:\n\n    iex> lens = Lens.multiple([Lens.at(0), Lens.at(1), Lens.at(3)])\n    iex> list = [0, 1, 2, 3, 4]\n    iex> Deeply.get_all(list, lens)\n    [0, 1, 3]\n    iex> Deeply.update(list, lens, & &1 * 1111)\n    [0, 1111, 2, 3333, 4]\n\nThis is essentially the implementation of `Lens2.Lenses.Indexed.indices/1`.","ref":"Lens2.Lenses.Combine.html#multiple/1"},{"type":"function","title":"Lens2.Lenses.Combine.recur/1","doc":"The [Lens 1](https://hexdocs.pm/lens/readme.html) name for `repeatedly/1`.\n\nIt confused me, so I retaliated by renaming it.","ref":"Lens2.Lenses.Combine.html#recur/1"},{"type":"function","title":"Lens2.Lenses.Combine.recur_root/1","doc":"The [Lens 1](https://hexdocs.pm/lens/readme.html) name for `and_repeatedly/1`.","ref":"Lens2.Lenses.Combine.html#recur_root/1"},{"type":"function","title":"Lens2.Lenses.Combine.repeatedly/1","doc":"Apply a `descender` lens repeatedly to replace one or more pointers with all the matching pointers below them, descending \"all the way into\" the container.\n\nGiven a pointer to within a container, `repeatedly(descender)`:\n\n1. Applies `descender` to get a list of places. Let's suppose there\n   are two such places, and call them `X` and `Y`.\n2. *Replaces* the original pointer (or pointers) with `X` and `Y`.\n3. Applies `descender` to `X` and `Y` to get, say, `X_1` and `Y_1`.\n4. *Appends* the new pointers, giving `[X, Y, X_1, Y_1]`.\n5. Repeats steps 3 and 4 until no new places are found.\n\n(If you want to retain the original place or places, rather than\nreplacing them, use `and_repeatedly/1`.)\n\nHere is an example. (For a less terse explanation, see <<<<>>>>>\n\nConsider this structure:\n\n    %{below:\n         %{..., below: ...}}\n\n`Lens.key?(:below)` will transform a pointer to the root into a\npointer to the first substructure:\n\n         %{..., below: ...}}\n\nThis function can use that lens to give you pointers all the places under a `:below` key:\n\n    iex> lens = Lens.repeatedly(Lens.key?(:below))\n    iex> tree = %{below:\n    ...>           %{value: 1, below:\n    ...>                         %{value: 2}}}\n    iex> Deeply.get_all(tree, lens)\n    [%{value: 2},\n     %{value: 1, below: %{value: 2}}\n     # Note that the original `tree` is not included.\n    ]\n\nGiven pointers to the levels, a later lens can point to values at all of those levels:\n\n    iex> lens = Lens.repeatedly(Lens.key?(:below)) |> Lens.key(:value)\n    iex> tree = %{below:\n    ...>           %{value: 1, below:\n    ...>                         %{value: 2}}}\n    iex> Deeply.update(tree, lens, & &1 * 111111)\n    %{below:\n       %{value: 111111, below:\n                        %{value: 222222}}}\n    ...>\n    iex> Deeply.get_all(tree, lens)\n    [2, 1]\n\n**Warning:** The lens *must* make a distinction between \"missing\"\nand `nil`.  If the `Lens2.Lenses.Keyed.key?/1` above were replaced\nwith `Lens2.Lenses.Keyed.key/1`, which does not distinguish between\na key whose value is `nil` and a missing key, the descent would\nnever finish, as:\n\n    iex> nil |> Deeply.get_all(Lens.key(:a))\n    nil\n\nI think this behavior is for compatibility with `Access`:\n\n    iex> nil |> get_in([:a])\n    nil\n\nNote: `Lens2.Lenses.Indexed.at/1` does not make a distinction\nbetween an index in range or an index containing `nil`, so it cannot\nbe used with this function.\n\n    iex> nil |> get_in([Access.at(3)])\n    nil\n\n`repeatedly/1` is a synonym for `recur/1`, the name in the original `Lens` package.","ref":"Lens2.Lenses.Combine.html#repeatedly/1"},{"type":"function","title":"Lens2.Lenses.Combine.root/0","doc":"Create an initial pointer from the whole of a container.\n\n**Rarely used**\n\nMost lenses consume pointers to create new pointers. However,\nsometimes you want to *add* those new pointers to the original\npointer. Use `root/0` to represent the original.\n\nThat's the difference between `repeatedly/1` and\n`and_repeatedly/1`: the latter uses `both/2` and `root/0` to add the\nroot pointer to what `repeatedly/1` produces.\n\n    defmaker and_repeatedly(descender) do\n      pointers_below = repeatedly(descender)\n      both(root(), pointers_below)\n    end","ref":"Lens2.Lenses.Combine.html#root/0"},{"type":"function","title":"Lens2.Lenses.Combine.seq/2","doc":"Given two lenses, apply the second to the results of the first.\n\nThis is the mechanism behind pipelining. This:\n\n    Lens.key(:a) |> Lens.key(:b)\n\n... is the same as this:\n\n    Lens.seq(Lens.key(:a), Lens.key(:b))","ref":"Lens2.Lenses.Combine.html#seq/2"},{"type":"function","title":"Lens2.Lenses.Combine.seq_both/2","doc":"Like `seq/2` but retains the pointers from the first lens.\n\n\nWhereas `seq/2` discards the pointers from the first lens after the\nsecond consumes them, `seq_both` retains them. It is literally defined as:\n\n    both(seq(lens1, lens2), lens1)\n\nThis is similar to `and_repeatedly`, except it stops after one application.\n\n    iex> map = %{a: %{c: 1},\n    ...>         b: %{c: 2}}\n    iex> lens = Lens.keys([:a, :b]) |> Lens.seq_both(Lens.key(:c))\n    iex> Deeply.get_all(map, lens)\n    [ 1, 2, %{c: 1}, %{c: 2} ]\n\nThis behavior means that `Lens2.Deeply.update/3` applies \"bottom\nup\". Consider summing a nested list with `Enum.sum/1`:\n\n    iex> nested = %{a: [1, [20, 30, 40], 5]}\n    iex> lens = Lens.seq_both(Lens.key(:a), Lens.at(1))\n    iex> Deeply.update(nested, lens, &Enum.sum/1)\n    %{a: 96}\n\n*First*, the interior list `[20, 30, 40]` is summed, resulting in\nthe list `[1, 90, 5]`. *Then* the summation function is applied to\nthat, producing the final value of `96`.\n\n`Lens2.Deeply.put/3` can be used, but it (in effect) ignores the deeper pointers:\n\n    iex> nested = [ [ 1 ] ]\n    iex> lens = Lens.seq_both(Lens.at(0), Lens.at(0))\n    iex> Deeply.put(nested, lens, :NEW)\n    [ :NEW ]\n\nPresumably, the `1` in the `[1]` sublist was replaced with `:NEW`\nbefore the entire sublist was itself replaced, but you can't tell.","ref":"Lens2.Lenses.Combine.html#seq_both/2"},{"type":"module","title":"Lens2.Lenses.Enum","doc":"Lenses that work on `Enumerable` and `Collectable` containers.","ref":"Lens2.Lenses.Enum.html"},{"type":"function","title":"Lens2.Lenses.Enum.all/0","doc":"Given a pointer to an enumerable, return a lens that points to all its elements.\n\n    iex> Deeply.get_all([0, 1, 2], Lens.all)\n    [0, 1, 2]\n    iex> Deeply.update([0, 1, 2], Lens.all, &inspect/1)\n    [\"0\", \"1\", \"2\"]\n\nNote that order is preserved for enumerables where that makes sense\n(lists, but not maps, for example).\n\n`update` and `put` always produce a list. Consider applying `all` to\na map. The values pointed to are `{key, value}` tuples:\n\n    iex> Deeply.get_all(%{1 => 100, 4 => 400}, Lens.all) |> Enum.sort\n    [{1, 100}, {4, 400}]\n\nUpdating the values can be done by composing `all/0` with `Lens2.Lenses.Indexed.at/1`:\n\n    iex> lens = Lens.all |> Lens.at(1)\n    iex> Deeply.update(%{1 => 100, 4 => 400}, lens, &inspect/1) |> Enum.sort\n    [{1, \"100\"}, {4, \"400\"}]\n\nThe result could be \"poured\" into a map using `Enum.into`:\n\n    iex> %{1 => 100, 4 => 400}\n    ...> |> Deeply.update(Lens.all |> Lens.at(1), &inspect/1)\n    ...> |> Enum.into(%{})\n    %{1 => \"100\", 4 => \"400\"}\n\nThere is also a lens-maker that has the same effect:\n\n    iex> lens = Lens.into(Lens.all |> Lens.at(1), %{})\n    iex> %{1 => 100, 4 => 400}\n    ...> |> Deeply.update(lens, &inspect/1)\n\nIn this particular case, it would work equally well to pipe into `into/1`:\n\n    iex> lens = Lens.all |> Lens.at(1) |> Lens.into(%{})\n    iex> %{1 => 100, 4 => 400}\n    ...> |> Deeply.update(lens, &inspect/1)\n    %{1 => \"100\", 4 => \"400\"}\n\nBut see the `into/1` documentation for why such piping is error-prone.\n\n`into/1` is used by datatype-specific lens-makers, such as those in\n`Lens2.Lenses.Keyed` and `Lens2.Lenses.MapSet`, to produce the right\nresult type for updates.\n\nNote the lens produced by `all/0` works on lists but not tuples, and\non maps but not structs. Neither of the latter two implement\nprotocol `Enumerable`.","ref":"Lens2.Lenses.Enum.html#all/0"},{"type":"function","title":"Lens2.Lenses.Enum.into/2","doc":"On `update` and `put`, puts the result into a given `Collectable`. No effect on `get_all`.\n\n\nHere's an example of using `into` to update the values of a `Range`\nand put them `into` a `MapSet`:\n\n     iex> Deeply.update(0..5, Lens.all |> Lens.into(MapSet.new), &inspect/1)\n     MapSet.new([\"0\", \"1\", \"2\", \"3\", \"4\", \"5\"])\n\nHowever, it's tricksy to use `into/1` in a pipeline. The above lens is more safely written\nas:\n\n     Lens.into(Lens.all, MapSet.new)\n\nYou can just take my word for it, but if you'd like to understand why, read on.","ref":"Lens2.Lenses.Enum.html#into/2"},{"type":"function","title":"The Why - Lens2.Lenses.Enum.into/2","doc":"Suppose we've successfully used the pipeline form above, but now we come upon\nranges within a map:\n\n     %{a: 0..2, b: 3..4}\n\nWe want to explode all the interior ranges into MapSets of strings to get something like\nthis:\n\n    %{a: MapSet.new([\"0\", \"1\", \"2\"]),\n      b: MapSet.new([\"3\", \"4\"])}\n\nCopy and paste the previous solution, prepend a\n`Lens2.Lenses.Keyed.map_values/0`, and we're golden, right?\n\n    iex> lens = Lens.map_values |> Lens.all |> Lens.into(MapSet.new)\n    iex> Deeply.update(%{a: 0..2, b: 3..4}, lens, &inspect/1)\n    %MapSet{map: %{{:a, [\"0\", \"1\", \"2\"]} => [], {:b, [\"3\", \"4\"]} => []}}\n\nUm, what?\n\nThe problem is that the `into/1` is the last thing done. It works on the *entire* updated\ncontainer. It, in effect, does this:\n\n    iex> updated =\n    ...>   Deeply.update(%{a: 0..2, b: 3..4},\n    ...>                 Lens.map_values |> Lens.all,\n    ...>                 &inspect/1)\n    %{a: [\"0\", \"1\", \"2\"], b: [\"3\", \"4\"]}\n    iex> updated |> Enum.into(MapSet.new)\n    %MapSet{map: %{{:a, [\"0\", \"1\", \"2\"]} => [], {:b, [\"3\", \"4\"]} => []}}\n\nIn our case, we want the `into` to take place on intermediate containers, so we\nneed to wrap the `into/2` around only the relevant parts of the pipeline, like this:\n\n    iex> lens = Lens.map_values |> Lens.into(Lens.all, MapSet.new)\n    iex> Deeply.update(%{a: 0..2, b: 3..4}, lens, &inspect/1)\n    %{a: MapSet.new([\"0\", \"1\", \"2\"]),\n      b: MapSet.new([\"3\", \"4\"])}\n\nAlternately, we could make the separation like this.\n\n    iex> lens = Lens.seq(Lens.map_values, Lens.all |> Lens.into(MapSet.new))\n    iex> Deeply.update(%{a: 0..2, b: 3..4}, lens, &inspect/1)\n    %{a: MapSet.new([\"0\", \"1\", \"2\"]),\n      b: MapSet.new([\"3\", \"4\"])}","ref":"Lens2.Lenses.Enum.html#into/2-the-why"},{"type":"function","title":"Lens2.Lenses.Enum.update_into/2","doc":"On `update` and `put`, puts the result into a given `Collectable`. No effect on `get_all`.\n\n`all/0` can be used on any Enumerable. However, unless you do\nsomething special, the result of `Deeply.update` will be,\nspecifically, a list. If you want to get back the original type, you\nhave to explicitly \"pour\" the list into it. For simple cases, you can use use\n`Enum.into/2`. For example, here's how to update a `MapSet`:\n\n    iex> container = MapSet.new([1, 2, 3])\n    iex> Deeply.update(container, Lens.all, & &1*11111) |> Enum.into(MapSet.new)\n    MapSet.new([11111, 22222, 33333])\n\nThat's a little awkward: you might want a `Lens2.Lenses.MapSet.all`\nthat does the `into` for you. More importantly, it doesn't work in\nthe middle of a pipeline. Consider this:\n\n    iex> container = %{outer: MapSet.new([%{inner: 1}, %{inner: 2}])}\n    iex> lens = Lens.key(:outer) |> Lens.all |> Lens.key(:inner)\n    iex> Deeply.update(container, lens, & &1*11111)\n    %{outer: [%{inner: 11111}, %{inner: 22222}]}\n\nBy the time the `update` has returned, it's too late to convert the\nlist into a `MapSet`.  Instead, we have to separate the part of the\npipeline that produces a list we want to pour into a `MapSet` from\nthe part that builds the return value that will contain the\n`MapSet`.\n\n     #                     vvvvvvvvvvvvvvvvvvvvvvvvvvvv makes the pourable list\n       Lens.key(:outer) |> Lens.all |> Lens.key(:inner)\n     # ^^^^^^^^^^^^^^^^ handles poured value\n\nThe \"before pouring\" is separated with `update_into`:\n\n    iex> lens =\n    ...>   Lens.key(:outer)\n    ...>   |> Lens.update_into(MapSet.new, Lens.all |> Lens.key(:inner))\n    iex> container = %{outer: MapSet.new([%{inner: 1}, %{inner: 2}])}\n    iex> Deeply.update(container, lens, & &1*11111)\n    %{outer: MapSet.new([%{inner: 11111}, %{inner: 22222}])}\n\nNotice that the `Collectable` poured into is the first argument. The\ndeprecated maker `into/2` takes them in the other order, which\nexperience has been shown tempts people to put it at the *end* of\nthe pipeline, which is wrong, but works for simple cases, causing\npuzzlement when what used to work (by accident) suddently stops\nworking. `update_in/2` avoids this temptation.\n\nThe \"update\" in `update_in/2` signals that the lens has effect only\non `update` (or `put`, which uses `update`). It does not interfere with `get_all`,\nwhich continues to return a list:\n\n    iex> lens =\n    ...>   Lens.key(:outer)\n    ...>   |> Lens.update_into(MapSet.new, Lens.all |> Lens.key(:inner))\n    iex> container = %{outer: MapSet.new([%{inner: 1}, %{inner: 2}])}\n    iex> Deeply.get_all(container, lens)\n    [1, 2]","ref":"Lens2.Lenses.Enum.html#update_into/2"},{"type":"module","title":"Lens2.Lenses.Filter","doc":"Lenses that reduce a set of pointers into a smaller set of pointers.\n\nConsider a use of, say, `Enum.filter`:\n\n    iex> [-2, -1, 0, 1, 2] |> Enum.filter(& &1 > 0)\n    [1, 2]\n\n\nFunctions in this module convert such a function into a lens that\nwill (eventually) apply the same filter to already-selected elements\nof a container.\n\n    iex> lens = Lens.keys([:a, :c]) |> Lens.filter(& &1 > 0)\n    iex> container = %{a: 1, b: 9, c: -1}\n    iex> Deeply.get_all(container, lens)\n    [1]\n    iex> Deeply.update(container, lens, & -1111 * &1)\n    %{a: -1111, b: 9, c: -1}","ref":"Lens2.Lenses.Filter.html"},{"type":"function","title":"Lens2.Lenses.Filter.filter/1","doc":"Returns a lens that focuses on a subset of elements focused on by the given lens that satisfy the given condition.","ref":"Lens2.Lenses.Filter.html#filter/1"},{"type":"function","title":"Lens2.Lenses.Filter.filter/2","doc":"","ref":"Lens2.Lenses.Filter.html#filter/2"},{"type":"function","title":"Lens2.Lenses.Filter.reject/2","doc":"Returns a lens that focuses on a subset of elements focused on by the given lens that don't satisfy the given\ncondition.","ref":"Lens2.Lenses.Filter.html#reject/2"},{"type":"module","title":"Lens2.Lenses.Indexed","doc":"Lenses specific to lists, plus one that works on both lists and tuples.\n\n`Deeply.put/3` and `Deeply.update/3` produce lists when applied to\nlists, tuples when applied to tuples. As always, `Deeply.get_all/2`\nproduces a list whether it operates on a list or a tuple.\n\nThese lenses do *not* work on `Enumerables` (except for lists).","ref":"Lens2.Lenses.Indexed.html"},{"type":"function","title":"Lens2.Lenses.Indexed.at/1","doc":"Returns a lens that points to the n-th element of a list or tuple.\n\n    iex>  lens = Lens.at(1)\n    iex>  Deeply.get_only({00, 10, 20}, lens)\n    10\n    iex>  Deeply.update([00, 10, 20], lens, & &1 / 10)\n    [00, 1.0, 20]\n    iex>  Deeply.put({\"my\", \"whiny\", \"tuple\"}, lens, \"favorite\")\n    {\"my\", \"favorite\", \"tuple\"}\n\nIndexes that are out of bounds are not allowed for tuples:\n    iex>  tuple = {1, 2, 3}\n    iex>  assert_raise(ArgumentError, fn ->\n    ...>    Deeply.get_all(tuple, Lens.at(-1))\n    ...>  end)\n    iex>  assert_raise(ArgumentError, fn ->\n    ...>    Deeply.get_all(tuple, Lens.at(3))\n    ...>  end)\n\nNegative indices *are* allowed for lists and have their usual meaning (count backwards):\n\n    iex> Deeply.put([0, 1, 2], Lens.at(-1), :NEW)\n    [0, 1, :NEW]\n\nIndexes beyond the end of the list are ignored. (This behavior is\nconsistent with `List.update_at/3` and `List.replace_at/3`.)\n\n    iex> Deeply.put([0, 1, 2], Lens.at(3), :NEW)\n    [0, 1, 2]","ref":"Lens2.Lenses.Indexed.html#at/1"},{"type":"function","title":"Lens2.Lenses.Indexed.back/0","doc":"Returns a lens that points after the last element of a list.\n\nSince there is nothing there, `Deeply.get_only/2` returns `nil`:\n\n   iex> Deeply.get_only([0, 1, 2], Lens.back)\n   nil\n\nHowever, you can use it to append to the list:\n\n   iex> Deeply.put([0, 1, 2], Lens.back, :NEW)\n   [0, 1, 2, :NEW]\n\nRaises an error if used on a tuple.","ref":"Lens2.Lenses.Indexed.html#back/0"},{"type":"function","title":"Lens2.Lenses.Indexed.before/1","doc":"Returns a lens that points before the given index, but after the prevous element.\n\nSince there is nothing there, `Deeply.get_only/2` returns `nil`:\n\n   iex> Deeply.get_only([0, 1, 2], Lens.before(2))\n   nil\n\nHowever, you can use it to insert into the list:\n\n   iex> Deeply.put([0, 1, 2], Lens.before(2), :NEW)\n   [0, 1, :NEW, 2]\n\nRaises an error if used on a tuple.","ref":"Lens2.Lenses.Indexed.html#before/1"},{"type":"function","title":"Lens2.Lenses.Indexed.behind/1","doc":"Returns a lens that points after the given index, but before the next element.\n\nSince there is nothing there, `Deeply.get_only/2` returns `nil`:\n\n   iex> Deeply.get_only([0, 1, 2], Lens.behind(0))\n   nil\n\nHowever, you can use it to insert into the list:\n\n   iex> Deeply.put([0, 1, 2], Lens.behind(0), :NEW)\n   [0, :NEW, 1, 2]\n\nRaises an error if used on a tuple.","ref":"Lens2.Lenses.Indexed.html#behind/1"},{"type":"function","title":"Lens2.Lenses.Indexed.front/0","doc":"Returns a lens that points before the first element of a list.\n\nSince there is nothing there, `Deeply.get_only/2` returns `nil`:\n\n   iex> Deeply.get_only([0, 1, 2], Lens.front)\n   nil\n\nHowever, you can use it to prepend to the list:\n\n   iex> Deeply.put([0, 1, 2], Lens.front, :NEW)\n   [:NEW, 0, 1, 2]\n\n`front` raises an error if used on a tuple.","ref":"Lens2.Lenses.Indexed.html#front/0"},{"type":"function","title":"Lens2.Lenses.Indexed.index/1","doc":"An alias for `at`.","ref":"Lens2.Lenses.Indexed.html#index/1"},{"type":"function","title":"Lens2.Lenses.Indexed.indices/1","doc":"Returns a lens that points to all of the supplied indices.\n\n   iex> lens = Lens.indices([0, 2])\n   iex> Deeply.get_all([00, 10, 20, 30], lens)\n   [00, 20]\n   iex> Deeply.put([00, 10, 20, 30], lens, :NEW)\n   [:NEW, 10, :NEW, 30]\n\nAlas, you cannot use a range to refer to an `Enum.slice/2` of indices.\n\nThe handling of out-of-bounds elements is consistent with\n`at/1`. Negative indices count from the end, and out-of-range\nindices are ignored.\n\n   iex> lens = Lens.indices([0, 2, -1, 40])\n   iex> Deeply.get_all([00, 10, 20, 30], lens)\n   [00, 20, 30, nil]\n   iex> Deeply.put([00, 10, 20, 30], lens, :NEW)\n   [:NEW, 10, :NEW, :NEW]\n\nRaises an error if used on a tuple.","ref":"Lens2.Lenses.Indexed.html#indices/1"},{"type":"module","title":"Lens2.Lenses.Keyed","doc":"Lenses helpful for working with structs, maps, and types implementing the `Access` behaviour.\n\nUnlike `Access`, these functions make no distinction between structs\nand lenses. All operate on both. `Lens2.Deeply.put/3` and\n`Lens2.Deeply.update/3` will return a plain map if given one. If\ngiven a struct, they will return a struct of the same type.\n\nThese lenses are available under the `Lens` alias when you `use Lens2`.\n\nThese lenses can be used on `Keyword` lists, but\nthe results may surprise you.\n\n1. Operations apply only to the first matching key:\n\n        iex>  use Lens2\n        iex>  keylist = [a: 1, other: 2, a: 3]\n        iex>  Deeply.get_all(keylist, Lens.key(:a))\n        [1]         # not [1, 3]\n        iex>  Deeply.get_all(keylist, Lens.keys([:a, :other]))\n        [1, 2]      # not [1, 2, 3]\n\n2. Update operations will produce maps rather than keyword lists.\n\nSee the `Lens2.Lenses.Keyword` module for an alternative.","ref":"Lens2.Lenses.Keyed.html"},{"type":"function","title":"Lens2.Lenses.Keyed.key/1","doc":"Returns a lens that points to the value of `key`.\n\n    iex>  lens = Lens.key(:a)\n    iex>  %SomeStruct{a: 1, b: 2} |> Deeply.put(lens, :NEW)\n    %SomeStruct{a: :NEW, b: 2}\n\nIf the key doesn't exist in the map, a `nil` will be used instead.\n\n    iex>  lens = Lens.key(:missing)\n    iex>  %{a: 1, b: 2} |> Deeply.put(lens, :NEW)\n    %{a: 1, b: 2, missing: :NEW}\n    iex>  %{} |> Deeply.update(lens, fn nil -> :NEW end)\n    %{missing: :NEW}\n\n`Lens2.Deeply.get_all/2` and `Lens2.Deeply.get_only/2` treats maps and\nstructs the same when it comes to missing keys. That is, a `KeyError` is *not*\nraised as it would be for `struct[:some_misslelled_key]`:\n\n    iex>  lens = Lens.key(:missing)\n    iex>  %{a: 1, b: 2} |> Deeply.get_all(lens)\n    [nil]\n    iex>  %SomeStruct{a: 1, b: 2} |> Deeply.get_all(lens)\n    [nil]\n\nLike `Map.put`, you can use this lens to break the\ncontract of a struct and add keys undeclared with `defstruct/1`:\n\n    iex>  %SomeStruct{a: 1, b: 2} |> Map.put(:missing, :NEW)\n    %{missing: :NEW, a: 1, __struct__: Lens2.Lenses.KeyedTest.SomeStruct, b: 2}\n\n    iex>  lens = Lens.key(:missing)\n    iex>  %SomeStruct{a: 1, b: 2} |> Deeply.put(lens, :NEW)\n    %{missing: :NEW, a: 1, __struct__: Lens2.Lenses.KeyedTest.SomeStruct, b: 2}\n\nDon't do that.\n\n`key!/1` is more appropriate for structs.","ref":"Lens2.Lenses.Keyed.html#key/1"},{"type":"function","title":"Lens2.Lenses.Keyed.key!/1","doc":"Returns a lens that points to the value of the given key. If the\nkey does not exist an error will be raised.\n\n    iex> lens = Lens.key!(:missing)\n    iex> %{a: 1} |> Deeply.put(lens, :NEW)\n    ** (KeyError) key :missing not found in: %{a: 1}","ref":"Lens2.Lenses.Keyed.html#key!/1"},{"type":"function","title":"Lens2.Lenses.Keyed.key?/1","doc":"Returns a lens that points to the value of the given key. If\nthe key does not exist it points to nothing.\n\nHere is the difference between this function and\n`Lens2.Lenses.Keyed.key/1` when it comes to retrieving values:\n\n    iex> %{a: 1} |> Deeply.get_all(Lens.key?(:missing))\n    []\n    iex> %{a: 1} |> Deeply.get_all(Lens.key(:missing))\n    [nil]\n\nThis function cannot be used to add missing values:\n\n    iex> %{a: 1} |> Deeply.put(Lens.key?(:missing), :NEW)\n    %{a: 1}\n    iex> %{a: 1} |> Deeply.put(Lens.key(:missing), :NEW)\n    %{a: 1, missing: :NEW}\n\nIt *can* however be used on a key that's *present* but has a `nil` value:\n\n    iex> %{here: nil} |> Deeply.put(Lens.key?(:here), :NEW)\n    %{here: :NEW}\n\nThis behavior differs from `Access.key/1` or the shorthand form\n`put_in(..., [..., :key, ...] ...)`.\nUse `key/1` for that.","ref":"Lens2.Lenses.Keyed.html#key?/1"},{"type":"function","title":"Lens2.Lenses.Keyed.key_path!/1","doc":"Concise representation for following keys into a container.\n\nIt's pretty common to construct a lens with a pipeline like this:\n\n```\n    iex> lens = Lens.keys!([:a, :b]) |> Lens.key!(3)\n    iex> map = %{a: %{(4-1) => :a3},\n    ...>         b: %{(2+1) => :b3}}\n    iex> Deeply.get_all(map, lens)\n    [:a3, :b3]\n```\n\nThat's common enough that I provide a lens maker that takes a list\nof key or key-lists to have the same effect:\n\n```\n    iex> lens = Lens.key_path!([[:a, :b], 3])\n    iex> map = %{a: %{(4-1) => :a3},\n    ...>         b: %{(2+1) => :b3}}\n    iex> Deeply.get_all(map, lens)\n    [:a3, :b3]\n```\n\nThis could be seen as a compromise between `Access`-style definition\nand the normal lens pipeline style.\n\nTODO: There should be a `key_path?` and a `key_path`.","ref":"Lens2.Lenses.Keyed.html#key_path!/1"},{"type":"function","title":"Lens2.Lenses.Keyed.keys/1","doc":"Returns a lens that points to the values of the given keys.\n\nIt has the same behavior as `Lens2.Lenses.key/1`, just for multiple keys at once.\n\n    iex>  map = %{a: 1, b: 2}\n    iex>  lens = Lens.keys([:a, :b, :missing])\n    iex>  map |> Deeply.get_all(lens) |> Enum.sort\n    [1, 2, nil]\n    iex>  map |> Deeply.update(lens, fn\n    ...>    nil -> :NEW\n    ...>    x   -> 1111 * x\n    ...>  end)\n    %{a: 1111, b: 2222, missing: :NEW}\n\nThe list can be empty, which gets nothing and updates nothing.\n\n    iex> lens = Lens.keys([])\n    iex> %{a: 1} |> Deeply.get_all(lens)\n    []\n    iex> %{a: 1} |> Deeply.put(lens, :NEW)\n    %{a: 1}\n\nThat's the same behavior as `Lens2.Lenses.Basic.empty/0`.","ref":"Lens2.Lenses.Keyed.html#keys/1"},{"type":"function","title":"Lens2.Lenses.Keyed.keys!/1","doc":"Returns a lens that points to the values of the given keys. If any of\nthe keys doesn't exist, `Lens2.Deeply` functions raise an error.\n\nIt has the same behavior as `key!/1`, just for multiple keys at once.\n\n    iex>  lens = Lens.keys!([:a, :b])\n    iex>  %{a: 1, b: 2, c: 3} |> Deeply.get_all(lens) |> Enum.sort\n    [1, 2]\n\n    iex>  lens = Lens.keys!([:a, :missing])\n    iex>  %{a: 1} |> Deeply.put(lens, :NEW)\n    ** (KeyError) key :missing not found in: %{a: :NEW}","ref":"Lens2.Lenses.Keyed.html#keys!/1"},{"type":"function","title":"Lens2.Lenses.Keyed.keys?/1","doc":"Returns a lens that points to the values of the given keys. If any of\nthe keys does not exist, the `Lens2.Deeply` functions ignore it.\n\n    iex>  map = %{a: 1, b: 2, c: 3}\n    iex>  lens = Lens.keys?([:a, :b, :missing])\n    iex>  map |> Deeply.get_all(lens) |> Enum.sort\n    [1, 2]\n    iex>  map |> Deeply.put(lens, :NEW)\n    %{a: :NEW, b: :NEW, c: 3}","ref":"Lens2.Lenses.Keyed.html#keys?/1"},{"type":"function","title":"Lens2.Lenses.Keyed.map_keys/0","doc":"Returns a lens that points to all keys of a map.\n\n    iex>  lens = Lens.map_keys\n    iex>  map = %{[1] => 1, [2] => 2}\n    iex>  Deeply.get_all(map, lens) |> Enum.sort\n    [[1], [2]]\n    iex>  Deeply.update(map, lens, fn [integer] ->\n    ...>    [integer * 1111]\n    ...>  end)\n    %{[1111] => 1, [2222] => 2}\n\nA lens produced by this function won't work on a struct. Updating\nstruct keys themselves (as opposed to their values) makes no\nsense. And I'm hoping you won't need to use `get_all` to get all the\nkeys of a struct, since all the keys are known at compile time.","ref":"Lens2.Lenses.Keyed.html#map_keys/0"},{"type":"function","title":"Lens2.Lenses.Keyed.map_values/0","doc":"Returns a lens that points to all values of a map or struct.\n\n    iex>  lens = Lens.map_values\n    iex>  map = %{a: 1, b: 2}\n    iex>  Deeply.get_all(map, lens) |> Enum.sort\n    [1, 2]\n    iex>  Deeply.put(map, lens, :NEW)\n    %{a: :NEW, b: :NEW}\n\n    iex>  lens = Lens.map_values\n    iex>  struct = %SomeStruct{a: 1, b: 2}\n    iex>  Deeply.get_all(struct, lens) |> Enum.sort\n    [1, 2]\n    iex>  Deeply.put(struct, lens, :NEW)\n    %SomeStruct{a: :NEW, b: :NEW}\n\nNote: This function is changed from its\n[`Lens`](https://hexdocs.pm/lens/readme.html) equivalent. That one\ndoes not work with structs.","ref":"Lens2.Lenses.Keyed.html#map_values/0"},{"type":"module","title":"Lens2.Lenses.Keyword","doc":"A variant of `Lens2.Lenses.Keyed` that applies to all matching `Keyword` keys.\n\nThe `Kernel` \"_in\" functions apply only to the *first* matching key in a `Keyword` list:\n\n    iex(3)> keylist = [a: 1, other: 2, a: 3]\n    [a: 1, other: 2, a: 3]\n    iex(4)> get_in(keylist, [:a])\n    1\n    iex(5)> put_in(keylist, [:a], 33)\n    [a: 33, other: 2, a: 3]\n    iex(6)> update_in(keylist, [:a], & &1 + 100)\n    [a: 101, other: 2, a: 3]\n\nLenses like `Lens2.Lenses.Keyed.key/1` mimic that behavior. (Indeed,\nthey're implemented using it.) Given that the Joe Q. Average lens\npoints at multiple places, that's counterintuitive. The lenses in this\nmodule will get, put, and update all matching keys.\n\n\n\nProblems: duplicate keys.\n\nSome set-type operations don't produce a keyword list.\n\n\nDeeply.put([a: 1], Lens.map_values, :NEW)\n%{NEW: 1}","ref":"Lens2.Lenses.Keyword.html"},{"type":"module","title":"Lens2.Lenses.MapSet","doc":"Lenses that work with `MapSet` structures.","ref":"Lens2.Lenses.MapSet.html"},{"type":"function","title":"Lens2.Lenses.MapSet.all/0","doc":"Like `Lens.all/0`, but intended for use with a `MapSet`.\n\nHere's how to update the `:a` values in a `MapSet` of maps:\n\n     iex> lens = Lens.MapSet.all |> Lens.key?(:a)\n     iex> container = MapSet.new([%{a: 1}, %{a: 2}, %{a: 3}])\n     iex> Deeply.update(container, lens, & &1*100)\n     MapSet.new([%{a: 100}, %{a: 200}, %{a: 300}])\n\n`all` also works reasonably well with `put`. Given the above\ncontainer and lens,\n\n     Deeply.put(container, lens, 3)\n\n... will produce:\n\n     MapSet.new([%{a: 3}]\n\nNote that the multiple maps with now-identical `:a` values have\nbeen correctly collapsed into one. (MapSets can't have duplicates.)\n\nAs normal, `get` functions return lists rather than MapSets:\n\n     Deeply.get_all(container, lens)\n     > [1, 2, 3]    # *not* MapSet.new([1, 2, 3])","ref":"Lens2.Lenses.MapSet.html#all/0"},{"type":"function","title":"Lens2.Lenses.MapSet.has!/1","doc":"Make a lens that points at elements containing a particular key/value pair.\n\nA `MapSet` might contain a struct or other `Access`-compatible container,\nsuch as a map or struct. A lens created by this maker will select all elements\nwith a given key that matches the given value:\n\n    iex> container = MapSet.new([%{name: \"bullwinkle\"}, %{name: \"rocky\"}])\n    iex> Deeply.get_all(container, Lens.MapSet.has!(name: \"rocky\"))\n    [%{name: \"rocky\"}]\n\nHere, I get all the names from a keyword list that are associated with the value\n`2`:\n\n    iex> container = [ [name: \"bullwinkle\", val: 1],\n    ...>               [name: \"rocky\",      val: 2],\n    ...>               [name: \"natasha\",    val: 2]] |> MapSet.new\n    iex> lens = Lens.MapSet.has!(val: 2) |> Lens.key(:name)\n    iex> Deeply.get_all(container, lens) |> Enum.sort\n    [\"natasha\", \"rocky\"]\n\nAs the exclamation point suggests, an error is raised if any of the\nelements doesn't have the given key.\n\nStructs are not required to implement the `Access` callback\nfunctions. (Indeed, even if it does implement `Access.fetch/2`,\nthat's not used.)","ref":"Lens2.Lenses.MapSet.html#has!/1"},{"type":"function","title":"Lens2.Lenses.MapSet.is/1","doc":"Make a lens that points at elements satisfying a predicate.\n\nHere is how you might increment all the even values of a `MapSet`:\n\n    iex> import Integer, only: [is_even: 1]\n    iex> container = MapSet.new([-1, 0, 1, 2])\n    iex> Deeply.update(container, Lens.MapSet.is(&is_even/1), & &1+1)\n    MapSet.new([-1, 1, 3])  # Note that duplicate 1 has been removed.","ref":"Lens2.Lenses.MapSet.html#is/1"},{"type":"extras","title":"Rationale","doc":"# Rationale\n\nI've written this rationale for people who are already familiar with\nElixir's built-in `Access` module and its related functions\n(`get_in/2`, `put_in/3`, `update_in/3`, etc.) If you're not familiar\nwith `Access`, you might want to read [\"For Access novices\"](for_access_novices.html), then\nskip to [Part 2](#part-2-why-a-new-package) on this page.","ref":"rationale.html"},{"type":"extras","title":"Part 1: Why not `Access`? - Rationale","doc":"Lenses are a tool for working with nested data structures, which I'll\ncall *containers*. Elixir already comes with a tool for that:\n\n     iex> container = %{top_level: [%{lower: 3}, %{lower: 4}]}\n     iex> put_in(container, [:top_level, Access.all, :lower], :NEW)\n     %{top_level: [%{lower: :NEW}, %{lower: :NEW}]}\n\nLenses provide an alternative:\n\n     iex> container = %{top_level: [%{lower: 3}, %{lower: 4}]}  # same as above\n     iex> lens = Lens.key(:top_level) |> Lens.all |> Lens.key(:lower)\n     iex> Deeply.put(container, lens, :NEW)\n     %{top_level: [%{lower: :NEW}, %{lower: :NEW}]}             # same as above\n\nThat looks like a lot of extra characters to achieve the same result. So: why lenses?\n\n1. Lenses have more power. That is, there are more specialized functions like\n   `Access.all/0`. Things that are awkward or impossible with just `Access`\n   are built in:\n   \n       # Increment all map values:\n       iex> Deeply.update(%{a: 1, b: 2, c: 3}, Lens.map_values, & &1+1)\n       %{c: 4, a: 2, b: 3}\n\n2. Lenses can work with types that don't implement the `Access` behaviour. For example,\n   `MapSets` don't have keys, so the `MapSet` module doesn't implement `Access.fetch/2`, so you can't\n   use `put_in/3`. But you can use `Deeply.put` when part of your nested container is a `MapSet`:\n   \n       iex> container = %{a: MapSet.new([%{aa: 1, bb: 2}])}\n       %{a: MapSet.new([%{bb: 2, aa: 1}])}\n       iex> lens = Lens.key(:a) |> Lens.MapSet.all |> Lens.map_values\n       iex> Deeply.put(container, lens, :NEW)\n       %{a: MapSet.new([%{bb: :NEW, aa: :NEW}])}\n   \nIn a way, what you just read is a lie. The list given to a function\nlike `get_in` is a description of how to navigate into a nested data\nstructure. An element in the list can be any function that has this interface:\n\n      fn\n        :get, container, continuation ->\n           ...\n        :get_and_update, container, get_and_update_function ->\n           ...\n      end\n\nSo you could write functions that obey that interface and descend just\nfine into MapSets. One way to look at lenses is that save you the\ntrouble of writing such functions:\n\n1. You can just use `Lens.MapSet.all` instead of writing it yourself.\n\n2. If you write a lot of such functions, you'll find that you're\n   repeating yourself. The traditional functional language approach to\n   repetition is to factor it out into smaller functions that you\n   compose together in different ways. But those functions have already been written.\n   They have names like `Lens.multiple` and `Lens.repeatedly`.","ref":"rationale.html#part-1-why-not-access"},{"type":"extras","title":"Part 2: Why a new package? - Rationale","doc":"There already exist lens packages at\n[hex.pm](https://hex.pm/packages?search=lens&sort=recent_downloads). I\nused [`Lens`](https://hexdocs.pm/lens/readme.html) and originally\nintended to add a few small things to it, but got carried away to the\npoint where a few pull requests wouldn't do it. Rather, a\nfork-and-extensive-reworking seemed justified. So I dubbed the\nprevious package \"Lens 1\" and have called this one \"Lens 2\". Here are the\nhighlights of the new package:\n\n1. Lens 2 is largely backwards compatible. All of the functions that\n   make lenses still exist and are used the same way:\n   \n   \n   ```elixir\n   iex> use Lens2\n   iex> lens = Lens.key(:clusters) |> Lens.map_values |> Lens.key!(:age)\n   ```\n\n3. However, in Lens 1, the functions that *make* lenses and the functions that\n   *use* them were in the same module, `Lens`. I've separated the latter out into\n   the `Deeply` module and changed some names to ones I think are\n   usefully closer to Elixir conventions. (So the lens version of\n   `update_in/3` is `Lens2.Deeply.update` rather than `Lens.map`.)\n   \n1. Lenses are notoriously hard to understand. A lot of that, I think,\n   comes down to documentation. While Lens 1 is better\n   than others I've seen, the amount of work it demands from the\n   learner is still too much, in my opinion.\n   So this package has lots of explanatory and tutorial documentation, longer docstrings,\n   and some renamed functions. (With the old names still supported.)\n   \n3. The `Deeply` API encourages information hiding and the use of structs\n   more than does Lens 1 (or `put_in` and friends). When I use\n   lenses, I usually make them part of a module's interface. For\n   example, I have a `Network` struct that connects named\n   \"clusters\". The connections are somewhat complicated internally,\n   but a client of `Network` needn't know that: it merely wants to\n   ask for one cluster's downstream neighbors. This function:\n   \n        Network.downstream_from(name)\n   \n   ... returns a lens for clients to use in calls like this:\n   \n        Deeply.get_all(network, Network.downstream_from(cluster))\n        \n   Often, that code could be simpler. `Network.downstream_from/1`\n   makes a lens that refers to a particular `cluster`. However,\n   lens-making functions often don't take parameters. For example, `Lens.map_values/0`\n   always refers to all the values of a map (or struct), and\n   `Network.linear_clusters/0` refers to all the \"linear clusters.\" In such a case, and\n   because `Network` is a struct, client code can fetch all the linear clusters with:\n   \n        Deeply.get_all(network, :linear_clusters)\n        \n   I like the idea of \"pointing at\" struct values with a single name or atom, without\n   client code having to care if the field is at the top level, is buried within the\n   network, or is computed on the fly. \n   \n   Although it's less emphasized in\n   functional programming than object-oriented programming, information hiding still rules. Joe Bergin\n   used to say of object-oriented code that \"I should be able to ask\n   you what you had for breakfast without knowing how to reach into\n   your stomach to find out\", and I think Lens 2 supports the\n   functional programming equivalent.\n   \n   \n4. It would be a tragic waste of human life for multiple people to\n   write lenses for `MapSet` or\n   [`BiMap`]([`BiMap`](https://hexdocs.pm/bimap/readme.html)), so this\n   is a place where that can be done once and for all.\n   \n   Moreover, although you can combine lenses to do strange and wonderful things,\n   a lot more people can *desire* some behavior than can easily *implement it*. So\n   such frequently-desired combinations can be shared from here.","ref":"rationale.html#part-2-why-a-new-package"},{"type":"extras","title":"For Access novices","doc":"# For `Access` novices\n\nLanguages like Elixir don't let you change data structures; instead,\nyou create a new data structure that's different from the\noriginal. That can be somewhat annoying when you want to change a\nsingle place within a deeply nested structure. From now on, I'm\ngoing to call such structures *containers*. We're interested in nested\ncontainers.\n\nHere's a simple example. We have `Network` structure. It has various\nfields, one of which is `:name_to_cluster`. That points to a `Map`\nwhose keys are atoms (cluster names) and whose values are `Cluster` structs. A\n`Cluster` has various fields, one of which, `:downstream`, holds a\n`MapSet` of atoms (also cluster names). Like this:\n\n![alt text coming](pics/tidy.png)\n\nWe want to add the value `:c` to the `MapSet` associated with the cluster named `:a`. \nThis code works:\n\n\n```elixir\n    new_cluster =\n      network.name_to_cluster[:a]\n      |> Map.update!(:downstream, & MapSet.put(&1, :c))\n\n    new_map =\n      network.name_to_cluster\n      |> Map.put(:a, new_cluster)\n\n    %{network | name_to_cluster: new_map}\n```\n\nFine, but what I'm doing has two conceptual steps:\n\n1. Point at the place you want to change.\n2. Cause the change.\n\nIn the above code, those two steps are buried inside a lot of\nbookkeeping code that does the work of constructing each level of the\nnew nested container. What really matters is highlighted below: the\npath into the container and the `MapSet.put` operation.\n\n![alt text coming](pics/tidy-scribbled.png)\n\n\nI want to describe the path and the update function, then sit back while the\ncompiler writes the bookkeeping code for me.\n\nYou probably know that Elixir offers the `Access` behaviour that does\njust that. Here's a better implementation of the above:\n\n```elixir\n    update_in(network.name_to_cluster[:gate].downstream,\n              & MapSet.put(&1, :some_name))\n```\n\nLooks pretty clear: every word is about either the path or the\nupdate to be done on whatever's at its end. Alternately, we can use\nsyntax that's a little longer but not as reliant on macro magic to\ndescribe the path:\n\n```elixir\n    path = [Access.key(:name_to_cluster), :gate, Access.key(:downstream)]\n    update_in(network, path,\n              & MapSet.put(&1, :some_name))\n```\n\nSo where do lenses come in?\n\nWell, let's say we want to add `:some_name` to *all* the clusters. I\nwas surprised that you couldn't do that with a single `update_in`\nbecause I thought `Access.all/0` would do it. Something like:\n\n```elixir\n    path = [Access.key(:name_to_cluster),\n            Access.all(),    # `all` will produce list of {key, value} tuples.\n            Access.elem(1)]  # Take the value\n```\n\nBut no joy:\n\n```\n** (RuntimeError) Access.all/0 expected a list, got: %{gate: %Cluster{downstream: MapSet.new([:big_edit, :has_fragments]), name: :gate}, watcher: %Cluster{downstream: MapSet.new([]), name: :watcher}}\n```\n\nTo handle multiple clusters, you need some kind of a loop, perhaps\nusing `for`. `for` does what I expected `Access.all` to do, and\nproduces tuples:\n\n\n```elixir\n    new_map =\n      for {name, cluster} <- network.name_to_cluster, into: %{} do\n        new_cluster = update_in(cluster.downstream, & MapSet.put(&1, :some_name))\n        {name, new_cluster}\n      end\n    %{network | name_to_cluster: new_map}\n```\n\nThere are a variety of other looping or recursive styles you could use. (Actually,\nthat's sort of a problem: it would be better to have one concise\nsolution than, say, four that are long enough you actually have to\nthink to write them or, sometimes, read them.)\n\n\nIn this lens package (and others like it), updating multiple\nclusters in the network doesn't require a loop:\n\n```elixir\n   lens = Lens.key(:name_to_cluster) |> Lens.map_values |> Lens.key(:downstream)\n                                        ^^^^^^^^^^^^^^^\n   Deeply.update(network, lens, & MapSet.update(&1, :some_name))\n```\n\nAs you'd hope, it looks almost the same to update a *subset* of the clusters:\n\n```elixir\n   lens = Lens.key(:name_to_cluster) |> Lens.keys([:gate, :watcher]) |> Lens.key(:downstream)\n                                        ^^^^^^^^^^^^^^^^^^^^^^^^^^^\n   Deeply.update(network, lens, & MapSet.update(&1, :some_name))\n```\n\nOr, if you prefer, you can use `update_in/3`, because lenses are\nfunctions that implement the behavior that `Access` requires;\n\n\n```elixir\n    path = [Access.key(:name_to_cluster),  # Note Access\n            Lens.keys([:gate, :watcher]),  # Note Lens\n            Access.key(:downstream)]\n    update_in(network, path, & MapSet.put(&1, :some_name))\n```\n\nThis would be more compelling, I guess, if I were using maps instead\nof structs. With maps, you can avoid `Access.key`:\n\n\n```elixir\n    path = [:name_to_cluster,\n            Lens.keys([:gate, :watcher]),\n            :downstream]\n    update_in(network, path, & MapSet.put(&1, :some_name))\n```\n\nI personally just use `Deeply` everywhere except when the\n`dot.and[bracket].notation` fits too well to pass up. I tend to avoid\nnested structures that expose the structure of their innards to all\ntheir clients, so that's not as often as you might guess.\n\nYou *can*, however, read this series as descriptions of how to create\ncustom path elements for `update_in` and friends (the way `Lens.keys`\nwas used above). You can do that directly by writing functions that\nmatch the required `Access` behaviour, but since lenses *are*\nfunctions that match that behaviour, maybe learning lenses is still\nworthwhile, if only for the greater variety of prepackaged functions.","ref":"for_access_novices.html"},{"type":"extras","title":"Are lenses for you?","doc":"# Are lenses for you?","ref":"are_lenses_for_you.html"},{"type":"extras","title":"It depends - Are lenses for you?","doc":"Lenses, like `Access`, are used for getting data from within nested\ncontainers, for putting data in nested containers, and for updating\nexisting data. They're part of the long tradition of methods to do\nCRUD (create/read/update/delete) that includes relational databases,\nHTTP verbs, and the like. Whether they're worth learning depends on\nyou and the work you do.\n\nThe out-of-the-box, batteries included, `Kernel` module functions\n`get_in/2`, `put_in/3`, `update_in/3`, and `pop_in/2` let you solve some CRUD\nproblems without writing an annoying amount of bookkeeping code: they allow\nshort and declarative(ish) solutions. For other CRUD problems, they don't.\n\nSometimes lenses will work better; they'll let you write straightforward\nsolutions when `update_in/3` and friends would require more convoluted\nones. The question is whether *sometimes* is *often enough*. Not in\nsome abstract sense, but *for you* (and your team) and for what you do.\n\nHow often do you get annoyed writing boilerplate code around nested\ndata structures? (Or, probably equally important, how often do you not\ncreate a nested structure because working with them is so annoying?)\n\nIs that often enough that it's worth the time to learn enough about lenses to\ndetermine whether they would make your life (and your team's life) better?\n\n(Note: sometimes `Access` can do things lenses won't. Most notably (to\nme), lenses don't support `pop_in/2`. I'll point out such deficiencies\nas this series goes along.)","ref":"are_lenses_for_you.html#it-depends"},{"type":"extras","title":"A note on choosing - Are lenses for you?","doc":"It may seem weird and offputting to say you should decide on\nlenses based on your personal and team annoyance with traversing nested\nstructures in a functional language. Surely I should be saying lenses\nare better in some absolute sense? Well...\n\nI got my first job as a programmer in 1975 (at the age of 16). I've\nseen a *lot* of programmers making judgments about technology new to\nthem. When they reach for generalizations about \"all programmers\" or\n\"all programs\", they tend to predict poorly. For example, from about 1983\n(when I was a [Common Lisp](https://en.wikipedia.org/wiki/Common_Lisp) implementer) to the early '90s, I heard an\nendless number of people say that garbage collection was impractical\nfor \"serious work\". Machines (except for\n[expensive custom hardware](https://en.wikipedia.org/wiki/Lisp_machine))\nobjectively just weren't fast enough, and would never *be* fast\nenough. Then Java came out, and the conventional wisdom completely\ndropped the issue, even for the slow machines of the day. Garbage\ncollection was now *assumed*: the question was how to use it most\nefficiently, except that most programmers never thought about that at\nall. They adopted the new [\"paradigm\"](https://en.wikipedia.org/wiki/Paradigm#Scientific_paradigm) and moved on.)\n\nAnd don't get me started about the debate (around 1981) of whether C\ncould ever replace assembly language for serious coding.\n\nIn any case: I've seen too many programmer decisions about technology\nthat are based on personal preference, often driven by what that\nprogrammer is used to, but – and let me emphasize this – **presented\nas an objective analysis**. To that, I repeat the last line of Ernest\nHemingway's\n[*The Sun Also Rises*](https://en.wikipedia.org/wiki/The_Sun_Also_Rises):\n\"Wouldn't it be pretty to think so?\"\n\nI *don't* think so, so I won't pitch lenses to you as if you were a\ndispassionate person optimizing some objective criterion. That is:\n\n1. If you are the exception, someone who really does weigh things objectively –\n   and I believe you might exist, you're just unusual: welcome!\n   The series will, I hope, give you enough information to rationally\n   judge. But I'm not going to structure my argument around your\n   criteria. Sorry!\n\n2. If you accept that you are prone to subjective judgments: welcome!\n   I hope to show you why my subjective judgment about lenses just\n   might mesh with yours, and that you might find lenses pleasant.","ref":"are_lenses_for_you.html#a-note-on-choosing"},{"type":"extras","title":"Efficiency - Are lenses for you?","doc":"In my\n**very** crude\nbenchmarking, lenses take about 1.5X the time of the equivalent\n`update_in/3` functions and around 2X of `get_in/2`. That's for a configuration of [maps\nand structs](file:///Users/bem/src/lens2/doc/Mix.Tasks.Timings.Keyed.html). For [lists](file:///Users/bem/src/lens2/doc/Mix.Tasks.Timings.List.htmlfile:///Users/bem/src/lens2/doc/Mix.Tasks.Timings.List.html), lenses are worse: about 2X `update_in` and 3X\n`get_in/2`. I imagine you could beat the `Access` functions with\nhand-crafted recursion (especially if you wrote it using\n[C](https://www.erlang.org/doc/apps/erl_interface/ei_users_guide) or\n[Rust](https://github.com/rusterlium/rustler) foreign\nfunctions).\n\n\nThe hoary old\nadage applies: you're probably better writing the code in the way\nmost readable to your team, then optimizing after \nbenchmarking shows you where the bottlenecks are.\n\nIt's worth calling out [Pathex](https://github.com/hissssst/pathex), a\nlens-like package said to be faster than `Access`.","ref":"are_lenses_for_you.html#efficiency"},{"type":"extras","title":"Migrating from Lens 1","doc":"# Migrating from Lens 1\n\nMigration is supposed to be easy.\n\nNote that you can migrate one module at a time.\n\n*There are probably gotchas yet to be discovered.*\n\n1. Put `use Lens2` at the top of any module that defines or uses\n   lenses. \n   \n2. Code that *makes* lenses (`Lens.key(:a)`) need not be changed.\n\n2. Replace the operations like `Lens.to_list` and `Lens.map` with their\n   `Lens2.Deeply` equivalents. Note that you have to change the\n   argument order:\n   \n        Lens.put(lens, container, value)     # Lens 1\n        Deeply.put(container, lens, value)   # Lens 2\n       \n   Here is a list of changes:\n   \n   \n        Lens.each(lens, container, f)         Deeply.each(container, lens, f)\n        Lens.get_and_map(lens, container, f)  Deeply.get_and_update(container, lens, f)\n        Lens.map(lens, container, f)          Deeply.update(container, lens, f)\n        Lens.one!(lens, container)            Deeply.get_only(container, lens) or Deeply.one!(container, lens\n        Lens.put(lens, container, v)          Deeply.put(container, lens, v)\n        Lens.to_list(lens, container)         Deeply.get_all(container, lens) or Deeply.to_list(container, lens)\n        \n4. `use Lens2` will import macros `deflens` and `deflens_raw`, so you\n   don't have to change them. However, it also imports `defmaker` and\n   `def_raw_maker`, which I prefer.\n\n3. If you use\n   [`TypedStruct`](https://hexdocs.pm/typedstruct/readme.html) and\n   [`TypedStructLens`](https://hexdocs.pm/typed_struct_lens/readme.html),\n   you have to\n   [I don't know what yet. Probably just adding `alias Lens2.TypedStructLens` to the head of the module].","ref":"migrating.html"},{"type":"extras","title":"Lenses move pointers","doc":"# Lenses move pointers\n\nThe word \"lens\" is a metaphor. We're supposed to gain some\nunderstanding of what this particular software construct does, based on our\nexperience with physical lenses in the real world.\n\nI never found the metaphor helpful when learning lenses. And since lenses\nhave the reputation of being hard to learn, I'm happy to abandon\nit (while keeping the name everyone uses). To me, a more helpful metaphor is the *pointer*. In the picture below,\nyou see someone using a slim wooden stick to point to something on a\nblackboard. The other picture shows an arrow – a pointer – pointing at a blob\nwith smaller blobs within it. These are – metaphorically – the same action.\n\n![Alt-text is coming](pics/tutorial01-pointer-comparison.png)[^1]\n\nThe right-hand image is of a data structure with smaller data\nstructures within it. I'm going to call the bigger structure a\n*container*. The contained structures may also be containers that have\ntheir own internal structure. Or they may be \"atomic\" values like integers or atoms.\n\nThe container has something pointing at it. Perhaps that's a variable\nit's bound to via pattern matching:\n\n```elixir\n{:ok, container} = make_that_container(...)\n```\n\nPerhaps the pointer\nindicates our container is part of a larger container, and the pointer represents\nthat you can go from the larger to the smaller, perhaps via constructs like these:\n\n```elixir\nlarger.smaller\nMap.get(larger, :smaller)\nEnum.at(larger, 3)\n```\n\n**What a lens does is transform one set of pointers to another.** It \ntakes a set of pointers to containers as its input. From them, it produces a new set of\npointers, typically to values within the original containers. Like this:\n\n\n![Alt-text is coming](pics/tutorial01-pointer-transformation.png)\n\nThe lens is the \"becomes\" in the picture: the creation of the five pointers on the\nright from the single pointer on the left. That is, a lens is a\n*function*:\n\n\n```elixir\niex> use Lens2     # sets up aliases `Lens` and `Deeply`\niex> Lens.at(3)\n#Function<1.126734921/3 in Lens2.Lenses.Indexed.at/1>\n```\n\n... and `Lens.at` is a function-making (or \"higher order\")\nfunction. I'm going to give such higher-order functions the name \"lens\nmakers\". It's kind of an ugly name, but it's important to remember the\ndifference between a lens function and the function that makes it.\n\nExcept when you're writing a new lens from scratch, your code never\ncalls a lens function directly. Instead, your code passes the lens to\nsome other function that does that work. As far as you're concerned,\nthe lens is no different than the `3` in `Enum.at(container, 3)` or\nthe `:a` in `Map.key(container, :a)`\n\nTo peek ahead, the functions that use lenses are in the `Lens2.Deeply` package (usually aliased as `Deeply`). Here's one:\n\n```elixir\niex> Deeply.update([0, 1, 2], Lens.at(1), & &1 * 1111)\n[0, 1111, 2]\n```\n\nI'm going to call such functions *operations* because they use lenses to operate on containers.","ref":"tutorial01-pointers.html"},{"type":"extras","title":"Lenses are all about \"zero, one, many\" - Lenses move pointers","doc":"Functions like `Map.get/3`, `Enum.at/2`, and so on are about a *single\nvalue* within a container. Their conceptual extension to functions\nlike `get_in/2` share that assumption. There are\nexceptions like `Access.all/0` or `Access.slice/1`, but I think it\nfair to say those are special cases: both conceptually and in common\nusage.\n\nLenses have a different base assumption: their code expects to be both\nconsuming and producing multiple pointers: maybe just one, maybe ten,\nmaybe even zero.\n\nLet's look at some examples.\n\nHere's a picture of a struct or a map or maybe a keyword list.\n\n![Alt-text is coming](pics/tutorial01-keyed.png)\n\n\nThe\ncontainer contains a set of `{key, value}` tuples. In the case of a\nkeyword list, that's the literal, concrete representation. Maps might have\na different internal structure, but the values within a map are always\npresented to the outside world as a tuple. Like this:\n\n```elixir\niex> for elt <- %{a: 1, b: 2}, do: IO.inspect(elt)\n{:a, 1}\n{:b, 2}\n```\n\nTo get a lens that converts a pointer-to-Map into pointers to all the values, you do this:\n\n```elixir\niex> lens = Lens.map_values\n```\n\n(All my examples implicitly `use Lens2`, which provides the `Lens`\nalias so that I don't have to write the function's real name,\n`Lens2.Lenses.Keyed.map_values/0`.)\n\nWhen it's used, the lens will produce these pointers for the container I showed above: \n\n![Alt-text is coming](pics/tutorial01-map-values.png)\n\nYou can use that lens to set every element in the container by\nwrapping the lens in a singleton list and passing it to\n`put_in/3`:\n\n```elixir\niex> map = %{a: 1, b: 2, c: 3, d: 4, e: 5}\niex> put_in(map, [lens], :NEW)\n                 ^^^^^^\n%{c: :NEW, a: :NEW, d: :NEW, e: :NEW, b: :NEW}\n     ^^^^     ^^^^     ^^^^     ^^^^     ^^^^\n```\n\nSimilarly, you can use the lens with `get_in/2` and `update_in/3`):\n\n```elixir\niex> get_in(map, [lens])\n[3, 1, 4, 5, 2]\niex> update_in(map, [lens], & &1 * 1111)\n%{c: 3333, a: 1111, d: 4444, e: 5555, b: 2222}\n```\n\n(I'll note here that `Access.all/0` can only be used on lists, so you\ncan't use out-of-the-box Elixir to do what we just did – except by coding manually the function that `Lens2.Lenses.Keyed.map_values/0` gives you for free. \n\nHaving to wrap the lens in a list is a little annoying, so you can use\nfunctions from `Lens2.Deeply` instead. (There are other reasons to use\nthem, which I'll cover later.)\n\nHere's `Lens2.Deeply.put/3`:\n\n```elixir\niex> Deeply.put(map, lens, :NEW)\n%{c: :NEW, a: :NEW, d: :NEW, e: :NEW, b: :NEW}\n```\n\n-----\n\n`Lens.map_values/0` points to all the map's values. Unsurprisingly,\nyou can point to just one value instead:\n\n```elixir\niex> lens = Lens.key(:c)\n```\n\n![Alt-text is coming](pics/tutorial01-key.png)\n\n\nAnd then you can update only `:c`'s value:\n\n```elixir\niex> Deeply.update(map, lens, & 1111111*&1)\n%{c: 3333333, a: 1, d: 4, e: 5, b: 2}\n```\n\nBut let's see what happens if we use this new lens with `get_in/2`:\n\n```elixir\niex> get_in(map, [lens])\n[3]\n```\n\nThat's weird. We wouldn't expect a list from this:\n\n\n```elixir\niex> get_in(map, [:c])\n3\n```\n\n... so why do we get it with a lens? It's because a lens used with\n`get_in` might point to multiple elements of the container, so it's\nalways working internally with lists. If it treated singleton lists\ndifferently (by unwrapping them), calling code couldn't distinguish\nbetween a single value that *is* a list or a list of multiple\nvalues. So everything stays in a list.\n\nTo remind you that you always get a list, the `Deeply` equivalent to\n`get_in/2` is named `Lens2.Deeply.get_all/2`. I apologize in advance\nfor the number of times you'll habitually type `Deeply.get(...)` and\nthen be annoyed by this compiler error:\n\n     Lens2.Deeply.get/2 is undefined or private. Did you mean:\n\n           * get_all/2\n           * get_only/2\n     \nNote `Lens2.Deeply.get_only/2`. If you're really truly sure you'll get\na singleton list, you can use it to unwrap the value for you. If it's\nnot a singleton list, `get_only` will raise an error.\n\n\n-----\n\nAs a final example, you can point at a subset of the values:\n\n![Alt-text is coming](pics/tutorial01-keys.png)\n\n\n```elixir\niex> lens = Lens.keys([:a, :e])\niex> Deeply.get_all(map, lens)\n[1, 5]\n\n```\n\nThis has *almost* the same behavior as `Map.take/2`, except the latter returns a map:\n\n```elixir\niex> Map.take(map, [:a, :e])\n%{a: 1, e: 5}\n```\n\n[^1]: The photo is via [Fagnar Brack](https://fagnerbrack.medium.com/personal-experience-doesnt-matter-cb060b42f99a). Found via a DuckDuckGo [\"free to share and use\" search](https://duckduckgo.com/?t=ffab&q=teacher+pointing+at+a+blackboard&atb=v196-1&iax=images&iai=https%3A%2F%2Fcdn-images-1.medium.com%2Fmax%2F2000%2F1*20JDS-KefsZFZoQlSqehaw.jpeg&ia=images&iaf=license%3AShare), although I couldn't find the specific license.","ref":"tutorial01-pointers.html#lenses-are-all-about-zero-one-many"},{"type":"extras","title":"Pointing into nested containers","doc":"# Pointing into nested containers\n\nLenses' claim to fame is the ability to descend through nested data\nstructures in a variety of ways, for a variety of structures. This\npage is about how you use them to do that.","ref":"tutorial02-nested.html"},{"type":"extras","title":"Composing lens makers - Pointing into nested containers","doc":"Lens-creating functions can be combined using the pipe (`|>`)\nmacro.\n\nSuppose I've `use`d `Lens2` and so can refer to lens-maker functions\nwith `Lens`. Here, again, are pictures showing the pointed-at values\nfrom applying, respectively, `Lens.key(:c)` and `Lens.key([:a, :e])`\nto two copies of the same five-element map.\n\n\n![Alt-text is coming](pics/tutorial02-two-maps.png)\n\nHowever, let's suppose the second map is actually embedded within the first, as the value of the key `:c`:\n\n\n![Alt-text is coming](pics/tutorial02-blended-maps.png)\n\nWe want the pointers into that nested map, specifically at the values of the `:a` and `:e` keys:\n\n\n![Alt-text is coming](pics/tutorial02-nested-pointers.png)\n\nThat's easy to do by composing the Lens-making functions:\n\n\n```elixir\niex>  use Lens2\niex>  lens = Lens.key(:c) |> Lens.keys([:a, :e])\n#Function<13.52599976/3 in Lens2.Lenses.Combine.seq/2>\n```\n\nLet's see the new composed lens at work.\n\n```elixir\niex>  map = %{a: 1, b: 2, c: 3, d: 4, e: 5}\n%{c: 3, a: 1, d: 4, e: 5, b: 2}\niex>  nested = %{map | c: map}\n%{c: %{c: 3, a: 1, d: 4, e: 5, b: 2}, a: 1, d: 4, e: 5, b: 2}\n\niex>  Deeply.get_all(nested, lens)\n[1, 5]\niex>  Deeply.put(nested, lens, :NEW)\n%{c: %{c: 3, a: :NEW, d: 4, e: :NEW, b: 2}, a: 1, d: 4, e: 5, b: 2}\n                ^^^^           ^^^^\n```","ref":"tutorial02-nested.html#composing-lens-makers"},{"type":"extras","title":"Another example: filtering pointers - Pointing into nested containers","doc":"Lenses don't have to descend into a data structure. Some lenses can\nremove pointers. For example, consider this map from numbers to names:\n\n     iex> map = %{1 => \"one\", 2 => \"two\", 3 => \"three\"}\n     \n`Lens.map_values` will transform a pointer to the map into pointers to the values:\n\n     iex> Deeply.get_all(map, Lens.map_values)\n     [\"one\", \"two\", \"three\"]\n\nWe can use `Lens2.Lenses.Filter.filter/1` (aliased to `Lens.filter/1`) to make a lens that retains only pointers to three-character names:\n\n     iex> lens = Lens.map_values |> Lens.filter(& String.length(&1) == 3)\n     \n... and use it to boost those names' visibility:\n\n    iex> Deeply.update(map, lens, & String.upcase(&1))\n    %{1 => \"ONE\", 2 => \"TWO\", 3 => \"three\"}","ref":"tutorial02-nested.html#another-example-filtering-pointers"},{"type":"extras","title":"How pipelining works - Pointing into nested containers","doc":"Normally, we think of a maker like `Lens.keys` as taking a single argument:\n\n    iex> Lens.keys[:a, :b]\n   \nHowever, each lens maker has a two-argument version whose first\nargument is a *lens* (not a lens *maker*). Therefore this pipeline:\n\n    iex> lens = Lens.map_values() |> Lens.keys([:a, :b])\n    \nis just syntactic sugar for this:\n\n    iex> lens = Lens.keys(Lens.map_values(), [:a, :b])\n    \nThe second variant doesn't explicitly appear in API documentation\nbecause the documentation for dozens of lens makers would just say\n\"This works the same as every other lens maker that takes a first\n`lens` argument.\"\n\n----\n\nIf you're like me, pipelining can trip you up. A few times I've had a lens defined for a particular\ndata structure, like this one for a two-level map:\n\n    iex> two_level_lens = Lens.key(:a) |> Lens.key(:aa)\n    \n... and then I have a list containing such maps and want a lens that will pick\nout all the `:aa` values of all the maps in the list:\n\n    iex> Deeply.get_all(list_of_maps, a_bigger_lens)\n    [1, 2]\n    \nThe question is: how to easily make `a_bigger_lens`? Because I know\nthat `Lens2.Lenses.Enum.all/0` will give me all elements of a list, a simple\ncomposition should do it:\n\n    iex> a_bigger_lens = Lens.all |> two_level_lens.()\n    \nBut oops:\n\n    ** (BadArityError) #Function<19.51288540/3 in Lens2.Lenses.Combine.seq/2>\n    with arity 3 called with 1 argument\n    (#Function<1.73076862/3 in Lens2.Lenses.Enum.all/0>)\n\nHere I'm passing a lens (constructed by the lens maker `Lens.all`) to\nanother *lens*, not to a lens maker. That is, I'm passing the new lens to the *result* of `Lens.key(:a) |> Lens.key(:aa)`, not – as I sometimes sloppily assume – to `Lens.key/1`. \n\n\nThe way to compose a new lens with an existing lens is to use `Lens2.Lenses.Combine.seq/2`:\n\n    iex> a_bigger_lens = Lens.all |> Lens.seq(two_level_lens)\n    # or, without the pipeline:\n    iex> a_bigger_lens = Lens.seq(Lens.all, two_level_lens)\n\nIn fact, the definition of the hidden two-argument version of, for\nexample, `Lens2.Lenses.Keyed.keys/1` uses `Lens.seq/2`:\n\n    def keys(previous_lens, key_list) do\n      Lens.seq(previous_lens, keys(key_list))\n    end","ref":"tutorial02-nested.html#how-pipelining-works"},{"type":"extras","title":"Defining a lens maker - Pointing into nested containers","doc":"There are two ways to define a lens maker: coding one up from scratch,\nor composing existing lens makers. The first way is rare and more\ncomplicated, so I'll put that off and talk only about composition.\n\nSuppose I frequently want to descend two levels into a nested map. Rather than write code like:\n\n    Lens.key(:level1_key) |> Lens.key(:level2_key)\n    \n... all over the place, I prefer to write a function that does the busywork for me:\n\n    MyLens.nested(:level1_key, :level2_key)\n    \nI could use a simple `def`:\n\n    def nested(level1, level2),\n        do: Lens.key(level1) |> Lens.key(level2)\n    \nThat works for making an isolated lens, but it doesn't work for composition:\n\n    iex> Lens.at(0) |> MyLens.nested(:a, :b)\n    ** MyLens.nested/3 is undefined or private. Did you mean:\n\n           * nested/2    \n\nThe problem is that I haven't defined the extra-argument version. I could do that easily enough:\n\n    def nested(lens, level1, level2),\n      do: Lens.seq(lens, nested(level1, level2)\n        \nHowever, that code will always always look the same, so there's a macro that defines both versions with a single definition:\n\n    defmaker nested(level1, level2),\n      do: Lens.key(level1) |> Lens.key(level2)\n\n(In the Lens 1 package,\nthis is called `deflens` and you can still use that name if you\nprefer. But I'm hoping the name will help you\nkeep in mind that you're not defining a *lens* but rather a lens\n*maker*, thus saving you from some mistakes. Also: in Lens 1, you\ndefine a lens maker from scratch with `deflens_raw`. I prefer\n`def_raw_maker/2`.)\n\nYou'll probably write a lot of lens makers of this sort.","ref":"tutorial02-nested.html#defining-a-lens-maker"},{"type":"extras","title":"Missing and nil values","doc":"# Missing and nil values\n\nIn Elixir, a `nil` sometimes means \"there is nothing here\" and\nsometimes \"there is something here, specifically `nil`.\n\n    iex> Map.get(%{a: nil}, :a)\n    nil\n    iex> Map.get(%{      }, :a)\n    nil\n\nSometimes you want to handle the two cases differently. For example:\n\n    iex> Map.put_new(%{a: 1}, :a, :NEW)\n    %{a: 1}\n    iex> Map.put_new(%{    }, :a, :NEW)\n    %{a: :NEW}\n\nThis page is how you navigate such distinctions when using lenses. Unlike\n`Map`, where you choose an operation (`Map.put/3`\nvs. `Map.put_new/3`), with lenses you choose a different lens maker.","ref":"tutorial04-missing.html"},{"type":"extras","title":"Keyed lenses (maps, structs, and `Access.fetch`) - Missing and nil values","doc":"Lenses for map structures come in three varieties, such as `Lens.key`,\n`Lens.key?`, and `Lens.key!`. (There are also `Lens.keys`, `Lens.keys?`, and `Lens.keys!`.)\n\n`Lens.key` treats a missing value and nil the same way:\n\n    iex> use Lens2\n    iex> Deeply.get_all(%{a: nil}, Lens.key(:a))\n    [nil]\n    iex> Deeply.get_all(%{      }, Lens.key(:a))\n    [nil]\n\n    iex> Deeply.put(%{a: nil}, Lens.key(:a), :NEW)\n    %{a: :NEW}\n    iex> Deeply.put(%{      }, Lens.key(:a), :NEW)\n    %{a: :NEW}\n    \n    iex> Deeply.update(%{a: nil}, Lens.key(:a), &inspect/1)\n    %{a: \"nil\"}\n    iex> Deeply.update(%{      }, Lens.key(:a), &inspect/1)\n    %{a: \"nil\"}\n    \nIf the container is a struct, `Deeply.get_all` behaves the same as for\na plain map. It will still produce a `nil` for a missing key.\n\n    iex> Deeply.get_all(%Point{}, Lens.key(:missing))\n    [nil]\n    \n`Deeply.put` with a missing key is alarming:\n\n    iex> Deeply.put(%Point{}, Lens.key(:missing), :NEW)\n    %{missing: :NEW, y: 2, __struct__: Point, x: 1}\n    \nWe've destroyed the contract for `Point`. This is, however, the same thing that `put_in/3` would do:\n\n    iex> put_in(%Point{x: 1, y: 2}, [Access.key(:missing)], :NEW)\n    %{missing: :NEW, y: 2, __struct__: Point, x: 1}\n    \nI assume there's a reason for that, but I don't know what it is.\n\n`Deeply.update` can also be used to add fields to a struct, as can `update_in/3`.","ref":"tutorial04-missing.html#keyed-lenses-maps-structs-and-access-fetch"},{"type":"extras","title":"key? - Missing and nil values","doc":"`Lens.key?` or `Lens.keys?`, in contrast, treat `nil` as a regular value, but handle\na missing value by doing nothing. For `Deeply.get_all`, nothing is\nincluded in the return list:\n\n    iex> Deeply.get_all(%{a: nil, b: 1}, Lens.keys?([:a, :b, :missing]))\n    [nil, 1] # `keys` would have provided an extra `nil`\n    \n`Deeply.put` will only override an existing value. (It's like a `put_not_new`.)\n\n    iex> Deeply.put(%{a: nil, b: 1}, Lens.keys?([:a, :b, :missing]), :NEW)\n    %{a: :NEW, b: :NEW}\n    \nFor `Deeply.update`, the update function is never called for a missing value.\n\n    iex> Deeply.update(%{a: nil, b: 1}, Lens.keys?([:a, :b, :missing]), &inspect/1)\n    %{a: \"nil\", b: \"1\"}\n\n`Lens.key?` works the same on structs as on plain maps: it cannot create missing values.","ref":"tutorial04-missing.html#key"},{"type":"extras","title":"key! - Missing and nil values","doc":"`Lens.key!` will raise an error whenever it detects any missing key.\n\n    iex> Deeply.get_all(%{}, Lens.key!(:missing))\n    ** (KeyError) key :missing not found in: %{}\n\n    iex> Deeply.put(%{a: 1}, Lens.keys!([:a, :missing]), :NEW)\n    ** (KeyError) key :missing not found in: %{a: :NEW}\n\n    iex(30)> Deeply.put(%{}, Lens.key!(:missing), &inspect/1)\n    ** (KeyError) key :missing not found in: %{}\n\nStructs are handled the same way.","ref":"tutorial04-missing.html#key"},{"type":"extras","title":"Other types - Missing and nil values","doc":"Any container that implements the `Access` behaviour will be treated like a map.\nMore precisely, \n\n* `Lens.key` uses `Access.fetch/2` but converts an `:error` return\n  value into `nil`. It uses `Access.get_and_update/3` to update (or\n  put) values.\n  \n* `Lens.key?` uses the same two functions but does nothing in the case\n  where `Access.fetch/2` returns `:error`.\n  \n* `Lens.key!` does the same, but it raises an error when\n  `Access.fetch/2` returns `:error`.","ref":"tutorial04-missing.html#other-types"},{"type":"extras","title":"Indexed lenses (lists, tuples) - Missing and nil values","doc":"The core function is `Lens2.Lenses.Indexed.at/1`. It and its\nderivative, `Lens2.Lenses.Indexed.indices/1`, will return `nil` when getting an\nindex that's out of bounds:\n\n      iex> Deeply.get_all([0, 1], Lens.at(2))\n      [nil]\n      iex> Deeply.get_all([0, 1], Lens.indices([0, 10000]))\n      [0, nil]\n      \nThis is consistent with the behavior of `Enum.at/2` and also `Lens.key`.\n\nWhen it comes to `Deeply.put` and `Deeply.update`, no change is made to an out-of-bound index:\n\n    iex> Deeply.put([0, 1], Lens.at(2), :NEW)\n    [0, 1]\n    iex> Deeply.update([0, 1], Lens.indices([0, 2]), &inspect/1)\n    [\"0\", 1]\n    \nThis is consistent with `List.replace_at/3`. One annoyance when using\n`Deeply.update` is that a `nil` (signifying \"missing\") is passed to the\nupdate function and *then* the return value is ignored. That's a\nproblem in the common case when the update function doesn't expect a nil:\n\n     iex> Deeply.update([\"0\", \"1\"], Lens.at(2), &Integer.parse/1)\n     ** (FunctionClauseError) no function clause matching in Integer.parse/2\n         The following arguments were given to Integer.parse/2:\n        \n             # 1\n             nil\n    \n\nThis is *not* consistent with the behavior of `update_in`, and I'm inclined to think it a bug. \n\n    iex> update_in([\"0\", \"1\"], [Access.at(2)], &Integer.parse/1)\n    [\"0\", \"1\"]","ref":"tutorial04-missing.html#indexed-lenses-lists-tuples"},{"type":"extras","title":"Tuples - Missing and nil values","doc":"`at` also works with tuples:\n\n    iex> Deeply.get_all({\"0\", \"1\", \"2\"}, Lens.indices([0, 2]))\n    [\"0\", \"2\"]\n    iex> Deeply.update({0, 1, 2}, Lens.at(2), & &1 * 1111)\n    {0, 1, 2222}\n\nHowever, you cannot use an index out of range:\n\n    iex> Deeply.get_all({\"0\", \"1\"}, Lens.at(2))\n    ** (ArgumentError) errors were found at the given arguments:\n\n            * 1st argument: out of range\n\nThis is consistent with `elem/2`.\n\nYou also get an `ArgumentError` when attempting to `put` or `update` a value out of range.","ref":"tutorial04-missing.html#tuples"},{"type":"extras","title":"Enumerable types - Missing and nil values","doc":"Although `Lens.at/1` is suggestive of `Enum.at/2`, you can't use it\nwith a non-list `Enumerable`. That makes sense for `put` and\n`update`, since there's no general way to modify elements of an\n`Enumerable`. Consider this:\n\n    iex> Deeply.put(0..5, Lens.at(1), 2)\n    ?????\n    \nWhat would that even mean?\n\nIt seems you should be able to use `Deeply.get_all`, but you can't\nbecause of\n[the lens implementation](implementation04-get_and_update.html).","ref":"tutorial04-missing.html#enumerable-types"},{"type":"extras","title":"Lenses specifically for adding to lists - Missing and nil values","doc":"`Lens2.Lenses.Indexed` supplies lenses that point, not to elements of a list,\nbut *next to them*. Consider `Lens2.Lenses.Indexes.before/1`:\n\n    iex> lens = Lens.before(2)\n    \nGiven a list like `[0, 1, 2]`, it lets you add a new element:\n\n    iex> Deeply.put([\"0\", \"1\", \"2\"], lens, :NEW)\n    [\"0\", \"1\", :NEW, \"2\"]\n\nIt's rather peculiar to ask for a value at the place where there isn't\na value. If you do, you'll get a `nil`:\n\n    iex> Deeply.get_all([\"0\", \"1\", \"2\"], Lens.before(2))\n    [nil]\n    \nSimilarly, you can use `Deeply.update`, but the update function will\nalways get a `nil`. Which is therefore a more elaborate version of\n`Deeply.put`:\n\n    iex> Deeply.update([\"0\", \"1\", \"2\"], Lens.before(2), fn nil -> :NEW end)\n    [0, 1, :NEW, 2]","ref":"tutorial04-missing.html#lenses-specifically-for-adding-to-lists"},{"type":"extras","title":"Non-list enumerables","doc":"# Non-list enumerables\n\n`Lens2.Lenses.Enum.all/0` is the building block for lens pipelines\nthat work with non-list `Enumerable` containers. \n\nTo start the explanation, here it is applied to a keyword list:\n\n    iex> use Lens2\n    iex> Deeply.get_all([a: 1, b: 2], Lens.all)\n    [a: 1, b: 2]\n    \nSince a keyword list is just a list of key/value tuples, the output looks the same as the input. \n\nHere's the same example for a map:\n\n    iex> Deeply.get_all(%{a: 1, b: 2}, Lens.all)\n    [a: 1, b: 2]\n    \nAgain, we get key-value pairs. To make that explicit:\n\n    iex> Deeply.get_all(%{1 => \"1\", 2 => \"2\"}, Lens.all)\n    [{1, \"1\"}, {2, \"2\"}]\n\nThis is the usual Elixir behavior when working with maps as enumerables:\n\n    iex> %{1 => \"1\", 2 => \"2\"} |> Enum.map(& &1)\n    [{1, \"1\"}, {2, \"2\"}]\n\nWe can pipe lens makers together to work on elements. Here's an\nimplementation of `Lens2.Lenses.Keyed.map_values/0` that uses `all` and `Lens2.Lenses.Indexed.at/1`:\n\n    iex> Deeply.get_all(%{a: 1, b: 2}, Lens.all |> Lens.at(1))\n    [1, 2]","ref":"tutorial04-non-list.html"},{"type":"extras","title":"Update - Non-list enumerables","doc":"That's not *quite* `map_values`, though, because of `update`:\n\n    \n    iex> Deeply.update(%{a: 1, b: 2}, Lens.map_values, & &1 * 111)\n    %{a: 111, b: 222}                 ^^^^^^^^^^^^^^^\n    iex> Deeply.update(%{a: 1, b: 2}, Lens.all |> Lens.at(1), & &1 * 111)\n    [a: 111, b: 222]                  ^^^^^^^^^^^^^^^^^^^^^^\n\n`all` will always produce a `List` on update. The solution is to add\nanother lens that \"pours\" the list into a `Collectable` in a manner\nanalogous to `Enum.into/2`. Lens 1 provided `Lens.into` for that:\n\n    iex> map_values = Lens.all |> Lens.at(1) |> Lens.into(%{})\n    iex> Deeply.update(%{a: 1, b: 2}, map_values, & &1 * 111)\n    %{a: 111, b: 222}\n\nThat exists in Lens 2 as well. Experience shows, though, that `Lens2.Lenses.Enum.into/2` is\nerror-prone. A pipeline like the above is a special case that's easy\nto mis-generalize from. After writing a whole lot of text explaining\nthe issue (see [here](Lens2.Lenses.Enum.html#into/2) *and*\n[here](draft_into.html)), I realized that I ought to [make the error harder to make](https://en.wikipedia.org/wiki/Poka-yoke). Hence `Lens2.Lenses.Enum.update_into/2`, which forces\nyou to make explicit which sub-pipeline produces the value to pour:\n\n    iex> map_values = Lens.update_into(%{}, Lens.all |> Lens.at(1))\n                      ^^^^^^^^^^^^^^^^\n    iex> Deeply.update(%{a: 1, b: 2}, map_values, & &1 * 111)\n    %{a: 111, b: 222}\n\nThis matters when you're updating a non-list `Enumerable` nested within a container, like this one:\n\n    iex> container = [%{}, %{a: 1, b: 2}, %{}]\n                           ^^^^^^^^^^^^^\n    iex> lens = Lens.at(1) |> Lens.update_into(%{}, map_values)\n                              ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n    iex> Deeply.update(container, lens, & &1 * 1111)\n    [%{}, %{a: 1111, b: 2222}, %{}]","ref":"tutorial04-non-list.html#update"},{"type":"extras","title":"Information hiding","doc":"# Information hiding\n\nI start from two beliefs:\n\n1. If you have complicated data, there's a reasonable chance your\n   first attempt at structuring it will be wrong. Or\n   adding new features will force restructuring.\n   \n   This is as true for\n   functional languages (with their CRUD emphasis) as it is for OO\n   languages.  (After all, the languages where ideas about\n   [modularity and information hiding took](https://prl.khoury.northeastern.edu/img/p-tr-1971.pdf)\n   root are at least arguably closer to modern functional languages\n   than to modern OO languages.)\n\n2. You want the inevitable restructuring to affect client code as\n   little as possible.\n\n   The fact that lenses are functions and\n   functional programming loves it some function composition suggests\n   that lenses make information hiding more idiomatic than `Access`\n   (which requires concatenating lists). \n   \nMy own practice (which is still evolving) is to give each struct their own lenses. Here's an example. \n\nI use [`TypedStruct`](https://hex.pm/packages/typedstruct) to define\nstructs, because it's prettier than `defstruct`. Here's an example.\n\n     defmodule Cluster do\n       typedstruct enforce: true do\n         field :name, atom\n         field :downstream, %{atom => MapSet.t(atom)}\n       end\n       ...\n\nIt seems reasonable to have a `downstream` lens:\n\n    defmaker downstream, do: Lens.key!(:downstream)\n\nI find this preferable to having client code flaunt its knowledge of\n`Cluster` structure with code like this:\n\n    Deeply.get_only(cluster, Lens.key!(:downstream))\n    \n... or the equivalent:\n\n    get_in(cluster, [Access.key(:downstream)])\n\nCode like this:\n\n    Deeply.get_only(cluster, Cluster.downstream)\n    \n... makes it less disruptive to change the `Cluster`\nstructure. Again: not a big deal with a shallowly nested container, but\nlenses would be a waste of time if all structures were shallow and\npredictable.\n\nIn fact, I like the CRUD abstraction enough that I've implemented shorthand\nfor cases where lens maker functions applied to structs take no arguments:\n\n    Deeply.get_only(cluster, :downstream)\n    \nAfter all, `cluster` contains its type, so the function call can be constructed easily enough.\n\nThere's something appealing to me about client code that says \"within\na cluster, there's a downstream. Fetch that, no matter where it is.\"\n\n----\n\nIn this style, containers that contain smaller containers will use the\nsmaller containers' lens makers to make their own lens makers. So suppose\nwe have a `Network` that contains `Clusters`:\n\n     defmodule Network do\n       typedstruct enforce: true do\n         field :clusters_by_name, %{atom => Cluster.t}\n         field :other_fields, any, default: :just_for_show\n       end\n       ...\n\nClients of `Network` will want to extract a particular named cluster's\ndownstream, which can be done like this:\n\n\n    defmaker downstream_of(cluster_name) do\n      Lens.key!(:clusters_by_name) |> Lens.key!(cluster_name) |> Cluster.downstream\n    end\n      \nThe important bit is that `Network` declines to make any guesses about\nthe structure of `Cluster`. It works from the lens API.\n\n----\n\nIn the code I'm thinking of as I write, it turned out\nthat a `Cluster` will have not just one `downstream` list, but a\nvariety of them. You see, Clusters send \"pulses\" to their downstream\nclusters. Originally, all pulses were the same, but later I needed to \ngive pulses \"types\", so that there's a distinction between a\ndefault pulse and a \"control pulse\". So suddenly, the `downstream` of\na cluster depends on the type of a pulse, so the code to retrieve the\ndownstream cluster names has to look like this:\n\n    Deeply.get_all(network, Network.downstream_of(originating_cluster_name, pulse_type))\n    \nThe important thing here is that, once we have the `cluster_names` function, client\ncode needn't care whether clusters contain a map of pulse types to\ndownstream names, meaning the path would be described like this:\n\n    defmaker downstream_of(originating_name, pulse_type do \n      Lens.key!(:clusters_by_name)\n      |> Lens.key!(originating_name)\n      |> Lens.key(:downstream_by_pulse_type)\n      |> Lens.key(pulse_type)\n    end\n\n... or whether `Network` maintains a mapping from `{cluster,\npulse_type`} to downstream names – removing from a `Cluster` any\nnotion of its \"downstream\" clusters. That would mean this path:\n\n    defmaker downstream_of(originating_name, pulse_type) do \n      Lens.key!(:downstreams)\n      |> Lens.key!({originating_name, pulse_type})\n    end\n\nThe point is: having lens makers be module functions makes change easier.","ref":"tutorial06-information-hiding.html"},{"type":"extras","title":"Sample pipelines","doc":"# Sample pipelines\n\nMost lenses are straightforward, and so are most pipelines of\nlenses. But lenses can be combined in some fairly complicated ways by\nusing lens makers in `Lens2.Lenses.Combine`.\n\n\nHere I want to call out a few that\noperate on other lenses and show examples of how they're used with\n`defmaker`.\n\n\nTO BE WRITTEN","ref":"tutorial05-combining.html"},{"type":"extras","title":"Introduction to debugging and def_raw_maker","doc":"# Introduction to debugging and `def_raw_maker`\n\n> I really hate this damn machine.    \n> I wish that they would sell it.    \n> It never does quite what I want,     \n> But only what I tell it.    \n> – Traditional    \n\nWhen using simple composed lenses like `Lens.key(:a) |>\nLens.keys([:aa, :bb])`, you don't have to understand what's happening\nbehind the scenes. The lens descends through key `:a`, then through\nkeys `:aa` and `:bb`, and either returns a list of values or an\nupdated complete container (depending on whether you're using `Deeply.get_all` or `Deeply.update`. \n\nBut sometimes – especially if you use some of the more unusual\ncombining lenses – you'll get surprised. I've reluctantly concluded\nthat the surest way to un-perplex yourself is to work through the\ndetailed steps of what's happening in the lens code. To do that, you\nneed to understand lenses well enough to write one from scratch. That's what this guide is about.\n\nNote: you might want to defer reading this page until you actually\n*are* perplexed or need to write a lens maker. Maybe you'll never need it!\n\n----\n\nA lens is an anonymous function created by a *lens maker* (a named\nfunction). You can easily write a lens maker using `def`, but the\n`def_raw_maker` macro handles some busywork for you. In either case, the\nform of the lens function is fixed: you use a template and fill in\nsome blanks.\n\nBecause the template is somewhat conceptually tricksy, I'll work my\nway up to its full glory by going through four versions:\n\n1. make a lens compatible with `Deeply.get_all`.\n2. make one compatible with `Deeply.update`. \n3. combine the two into a lens function that works with both.\n4. add compatibility with `Access` (so lenses work with `get_in`, `update_in`, and friends).\n\nThen I'll show how this understanding can be used to debug a couple of\nlenses that have surprising behavior in pipelines. \n\nFinally, I'll describe the implementations of some interesting makers,\nso that you can pattern your own makers off them, should you want your\nown makers.","ref":"implementation01-intro.html"},{"type":"extras","title":"Continuation-passing style","doc":"# Continuation-passing style\n\nIt's easier to understand the implementation of lenses if you first\nunderstand\n[\"continuation-passing style\"](https://en.wikipedia.org/wiki/Continuation-passing_style). Lenses\ndon't *exactly* use continuation-passing style, but it's pretty close.\n\nIn this style, every function call is a two-part instruction:\n\n1. Dear function, do that thing you do, using these arguments, then...\n\n2. ... do *not* return the result. Instead, pass it to the function I\n   also gave you in an argument. That function is called the\n   *continuation* (because it continues the overall computation of\n   which the function call is a part).\n\nAny function can be converted to continuation-passing style. Here's a\ncontinuation-passing version of `Map.put/3`:\n\n    def map_put(map, key, value, continuation) do\n      Map.put(map, key, value)\n      |> continuation.()\n    end\n    \n(You can find the code and tests quoted on this page at\n[`continuation_passing_test.exs`](../test/mostly_words/tutorial/continuation_passing_test.exs).)\n    \nAnd here's a use of `map_put`:\n\n    iex> map_put(%{}, :a, 1, & &1)\n    %{a: 1}\n    \nHere's a more elaborate call that puts two\ndifferent key/value pairs into a map:\n\n    iex> map_put(%{}, :a, 1,\n                 fn just_created_map ->\n                   map_put(just_created_map, :b, 2,\n                           & &1)\n                 end)\n    %{a: 1, b: 2}\n\nThere are two continuations here: one that calls another instance of\n`map_put`, and one that just returns the final value. The sequence of events is:\n\n1. `map_put` calculates %{a: 1}, and passes it to the bigger continuation.\n2. The continuation passes that value on to another `map_put`,\n   together with a second continuation, the identity function.\n3. The second `map_put` calculates %{a: 1, b: 2} and passes it to the second continuation.\n4. The second continuation just returns its value, so...\n5. ... the second `map_put` returns the value to...\n6. ... the first continuation, which returns it to...\n7. ... the first instance of `map_put` which...\n8. ... returns it to IEX for printing to the terminal.\n\n(A sufficiently smart compiler could eliminate all but the last\nreturn; indeed, continuation-passing style was invented for use in a\ncompiler as an intermediate form that lent itself to certain\noptimizations.)\n\nThe code isn't what you'd call wildly readable. There's a general structure that's obscured by a bunch of constants. Here's the code with the constants marked:\n\n      map_put(%{}, :a, 1,\n      ^^^^^^^ ^^^  ^^  ^\n              fn just_created_map ->\n                map_put(just_created_map, :b, 2,\n                ^^^^^^^                   ^^  ^\n                        & &1)\n                        ^^^^\n              end)\n\nCode full of constants can be turned into a function that takes appropriate arguments. Let's pull `%{}` and `& &1`out first:\n\n    two_puts =\n      fn initial_value, final_continuation ->\n         ^^^^^^^^^^^^^  ^^^^^^^^^^^^^^^^^^\n        map_put(initial_value, :a, 1,\n                ^^^^^^^^^^^^^\n                fn just_created_map ->\n                  map_put(just_created_map, :b, 2,\n                          final_continuation)\n                          ^^^^^^^^^^^^^^^^^^\n                end)\n      end\n\n    iex> two_puts.(%{}, & &1)\n    %{a: 1, b: 2}\n\nNext let's extract the two explicit calls to `map_put` into two \"step\"\narguments that give functions for the internal code to execute:\n\n    step_combiner =\n      fn step1, step2 ->\n         ^^^^^  ^^^^^\n        fn initial_value, final_continuation ->\n          step1.(initial_value,\n          ^^^^^  fn just_created_map ->\n                   step2.(just_created_map,\n                   ^^^^^  final_continuation)\n                 end)\n        end\n      end\n\nThe `step_combiner` is a function that returns a function that\nexecutes two computations in a row, passing the first result to the\nsecond computation. It could be used like this:\n\n    two_puts =\n      step_combiner.(\n        fn map, continuation ->\n          map_put(map, :a, 1, continuation)\n        end,\n        fn map, continuation ->\n          map_put(map, :b, 2, continuation)\n        end)\n\n    iex> two_puts.(%{}, & &1)\n    %{a: 1, b: 2}\n\n\nLet's do a little cleanup and use named functions instead of anonymous functions. Let's make the steps with this function:\n\n\n    def make_put_fn(key, value) do\n      fn map, continuation ->\n        Map.put(map, key, value) |> continuation.()\n      end\n    end\n\n(I expanded out `map_put` because I won't be using it any more.)\n\nLet's also make `step_combiner` a named function:\n\n    def step_combiner(step1, step2) do\n      fn initial_value, final_continuation ->\n        step1.(initial_value,\n               fn step2_value ->\n                 step2.(step2_value,\n                        final_continuation)\n               end)\n      end\n    end\n\nPutting that together, we get:\n\n    iex> step1 = make_put_fn(:a, 1)\n    iex> step2 = make_put_fn(:b, 2)\n\n    iex> put_twice = step_combiner(step1, step2)\n    iex> put_twice.(%{}, & &1)\n    %{a: 1, b: 2}\n\nSay, that looks kind of familiar:\n\n    iex> lens1 = Lens.key(:a)\n    iex> lens2 = Lens.key(:b)\n    iex> two_step = Lens.seq(lens1, lens2)\n    iex> Deeply.put(%{a: %{b: 2}}, two_step, :NEW)\n    %{a: %{b: :NEW}}\n\nIn fact, I can make it even more familiar. First, a function that\nlaunches the combined step and supplies the identity function:\n\n    def do_to(structure, step) do\n      step.(structure, & &1)\n    end\n\nSecond, a variant `make_put_fn` that takes a previous step and combines it with the one being created:\n\n    def make_put_fn(previous, key, value) do\n      step_combiner(previous, make_put_fn(key, value))\n    end\n\nAnd now:\n\n    iex> two_step = make_put_fn(:a, 1) |> make_put_fn(b: 2))\n    iex> do_to(%{c: 3}, two_step)\n    %{a: 1, b: 2, c: 3}","ref":"implementation02-continuation-passing.html"},{"type":"extras","title":"Version 1: get_all","doc":"# Version 1: `get_all`\n\nAlthough the previous page showed that lenses have a family\nresemblance to continuation-passing style, there are some differences:\n\n* I had a single \"launcher\" function, `do_to` that is reminiscent of\n  `Deeply.put`. But lenses have several launcher functions for different\n  purposes. That is, a single lens can accommodate `get_all`, `put`, and\n  `update` functions.\n\n* Put a bit differently, `make_put_fn` can only be used to put\n  values. What's being put is defined in the function that's like a\n  lens maker, rather than in the launcher function (like `Deeply.put`).\n\nThis page is a first step toward showing how lenses accomplish those\ndifferences. It starts by adding just just a smidge onto the\ncontinuation-passing style example.\n\nSince there will be multiple implementations of lenses coming up, I'll\ndistinguish them by using a version number prefix instead of\n`Lens`. Code on this page is version `V1`, and it will define\n`V1.at/1`, `V1.seq/2`, and `V2.all/0`, as well as a `Deeply`-style\n`get_all` function. The normal `Deeply` operations only work with the\n`V4` implementation, so each version will have its own operations\ndefined in a module I can't resist naming\n[Derply](https://knowyourmeme.com/memes/derp).\n\nThe code and tests for version 1 can be found in\n[`implementation_v1_get_test.exs`](../test/mostly_words/tutorial/implementation_v1_get_test.exs).\n\n## `V1.at`\n\nIf we're using continuation-passing style as a model, a lens should\ntake a container as an argument, plus a continuation-ish function. I\nsay \"continuation-ish\" because, while the argument has the effect of\ncontinuing a computation by descending more deeply into a container, a\nproper continuation is the last thing a function does. A lens\nfunction takes the return value of the continuation-ish function and does something with\nit. So I'm going to call that argument a *descender* instead of a continuation. \n\nHere is the definition of `V1.at/1`:\n\n    def at(index) do\n      fn container, descender ->\n        gotten =\n          Enum.at(container, index)\n          |> descender.()\n        [gotten]            # <<<<<\n      end\n    end\n\nThe difference is that the `descender`'s return value is wrapped in a list: \nthat's the contract a lens must follow. There has to be a way to\ndistinguish between returning a list of values and a single value\nthat's a list. That's done by wrapping everything in a list, so that a\nsingle value that's a list is returned like this:\n\n    [ [0, 1, 2] ]\n    \n... which is distinct from two values that are lists:\n\n    [ [0, 1, 2], [3, 4, 5] ]\n    \n... or six independent values (as you might get from `Lens2.Lenses.Enum.all/0`):\n\n    [ 0, 1, 2, 3, 4, 5 ]\n\nThe difference between `V1` and continuation-passing style comes down\nentirely to the little bit of code that executes after the descender.\n\n(Note that this version of `at` only works with lists, whereas the\nreal one also works with tuples. Nothing informative about lenses\nwould be gained by dragging in tuples, so I won't.)","ref":"implementation02-get_all.html"},{"type":"extras","title":"Derply.get_all - Version 1: get_all","doc":"As with the previous page's `do_to` function, `get_all` says what the last continuation in a chain should do. And that is... nothing: just return the value handed it back up the chain. \nSo the implementation is simple:\n\n    def get_all(container, lens) do\n      getter = & &1    # Just return the leaf value\n      lens.(container, getter)\n    end\n\nNow this works: \n\n    iex> Derply.get_all([\"0\", \"1\"], V1.at(1))\n    [\"1\"]","ref":"implementation02-get_all.html#derply-get_all"},{"type":"extras","title":"V1.seq - Version 1: get_all","doc":"As you've seen (I hope) a couple of times in this documentation, piping the lens from one\nlens maker into another makes use of `seq`. That is, this:\n\n    Lens.at(1) |> Lens.at(2) \n    \n... means that the second maker should produce code equivalent to this:\n\n    Lens.seq(Lens.at(1), Lens.at(2))\n    \nWe can use the previous page's `step_combiner` as a template. Instead\nof `step1` and `step2`, there'll be `outer_lens` and `inner_lens`:\n\n     def seq(outer_lens, inner_lens) do\n       fn outer_container, inner_descender ->\n         ...\n       end\n     end\n\nThe `outer_descender` has to be constructed by `seq`. I'll give it an explicit name:\n\n     def seq(outer_lens, inner_lens) do\n       fn outer_container, inner_descender ->\n         outer_descender =                                 # <<                             # << \n         outer_descender =                                 \n          fn inner_container ->                            \n            inner_lens.(inner_container, inner_descender)  \n          end                                              \n       \n        gotten =\n          outer_lens.(outer_container, outer_descender)\n        ????.(gotten)\n        ^^^^^^^^^^^^^\n       end\n     end\n     \nWhat? Consider this container: `[ [], [1, 2, 3] ]` and the\npipeline from `V1.at(1)` to `V1.at(2)`. \n\n1. `V1.at(1)`'s lens first uses `Enum.at(..., 1)` to extract\n   `[1, 2, 3]`, which it passes to the `outer_descender` and thus to\n   the lens from `V1.at(2)`.\n\n2. `V1.at(2)`'s lens extracts `2` and passes it to the inner descender,\n   which immediately returns it.\n   \n3. Now, the `V1.at(2)` lens wraps the result in a list, and returns it\n   to the `V1.at(1)` lens function. That function's `descender` returns it. \n   \n   At this point, we can represent the state of the `V1.at(1)` lens function as this:\n   \n   \n        fn container = [ [], [1, 2, 3] ], descender ->\n          gotten =\n            # Enum.at(container, 1)\n            # |> descender.()\n            [2]\n            ^^^\n          [gotten]\n        end\n\n4. As night follows day, the function will wrap `[2]` in a list, and\n   so return `[[2]]` to `seq`, which I'll represent as:\n   \n        fn outer_container, inner_descender ->\n          outer_descender = ...\n\n          gotten =\n            # outer_lens.(outer_container, outer_descender)\n            [[2]]\n            ^^^^^\n          ????(gotten)\n        end\n\n   We've doubly-wrapped the return value. Returning it would violate the\n   lens contract. Unwrapping could be done in several ways, but this is\n   the right one:\n\n        fn outer_container, inner_descender ->\n          outer_descender = ...\n          gotten = ...\n\n          Enum.concat(gotten)\n        end\n        \n   Why that instead of, say, this:\n\n          [gotten] =\n            # outer_lens.(outer_container, outer_descender)\n\n          gotten\n          \n   Well...","ref":"implementation02-get_all.html#v1-seq"},{"type":"extras","title":"The garden of forking paths - Version 1: get_all","doc":"> With apologies to [Jorge Luis Borges](https://en.wikipedia.org/wiki/The_Garden_of_Forking_Paths)\n\nThe previous example was linear: the code descended to a single\n\"leaf\" node, then returned the value found, wrapping and unwrapping as\nneeded. But lenses are built on the assumption that a single `Deeply`\noperation may require the lenses to descend to a leaf, retreat to some\nintermediate position in the container, descend again to another leaf,\nretreat again, and so on.\n\nHere's a simple example:\n\n    iex> nested = [ [0, 1, 2], [0, 1111, 2222]] \n    iex> lens = V1.all |> V1.at(1)\n    iex> Derply.get_all(nested, lens)\n    [1, 1111]\n    \nThe `Enum.concat/1` call in `seq` is what produces that result. Let's\nstep through that, meaning I need to shgow you the code for `V1.all`. It's simple:\n\n    def all do\n      fn container, descender ->\n        for item <- container, do: descender.(item)\n      end\n    end\n\nIn our example, `all` will do this, in effect:\n\n        for item <- [ [0, 1, 2], [0, 1111, 2222]],\n          do: descender.(item)\n      \n... which is the same as this:\n\n        [\n          descender.([0, 1, 2])\n          descender.([0, 1111, 2222])\n        ]\n        \nSince the `descender` calls `at(1)`, that's equivalent to this:\n\n        [\n          [1],\n          [1111]\n        ]\n        \n... and *that's* why `seq` uses `Enum.concat/1`:\n\n        iex> Enum.concat([ [1], [1111] ])\n        [1, 1111]\n\nThe upshot of all this is that unless you're writing a special lens\nmaker like `V1.seq`, you won't have to worry about any unwrapping or\nrewrapping. Just follow two rules:\n\n1. If you're fetching exactly one element, wrap it.\n2. If you're fetching zero to many elements, you've probably already got a list. Just return it. \n\n`V1.at` is an example of the first rule. `V1.all` is an example of the second.\n\nNow let's implement `Derply.update` and lenses that will work with that.","ref":"implementation02-get_all.html#the-garden-of-forking-paths"},{"type":"extras","title":"Version 2: update","doc":"# Version 2: update\n\nThe code and tests for this version – version 2 – can be found in\n[`implementation_v1_update_test.exs`](../test/mostly_words/tutorial/implementation_v2_update_test.exs).\n\nUpdate-capable lens code has the same \"shape\" as get-capable code. For\nexample, here's `V2.update` vs. `V1.get_all`. `update` takes an update\nfunction and uses it as the final \"descender\":\n\n\n    def update(container, lens, update_fn) do      def get_all(container, lens) do\n                                ^^^^^^^^^             \n      lens.(container, update_fn)                    lens.(container, & &1)\n                       ^^^^^^^^^                                      ^^^^\n    end                                            end\n\n(A `put` operation is the same, except that it codes up a constant-returning update function:\n\n    def put(container, lens, constant) do\n      lens.(container, fn _ -> constant end)\n    end\n\nI won't mention `put` any more.)\n\nIt's worth emphasizing what happens in a call like:\n\n    iex> lens = Lens.key(:a) |> Lens.key(:b) |> Lens.key(:c)\n    iex> container = %{a: %{b: %{c: 1}}}\n    iex> update_fn = & &1 * 1111\n    iex> Deeply.update(container, lens, update_fn)\n    %{a: %{b: %{c: 1111}}}\n    \nThe code descends all the way to the inner map `%{c: 1}`. It calls this function:\n\n    Map.update!(%{c: 1}, :c, update_fn)\n    \n... which produces `%{c: 1111}`. Having descended as far as it can, it\nbegins to \"retreat\" back to the original container. That means that\nthe lens for `key(:b)` will have pointers to two pieces of data:\n\n    %{b: %{c: 1}}  # embedded within the original container\n    %{c: 1111}     # a value returned from the lower lens\n\nIt must do this: \n\n    Map.put(%{b: %{c: 1}}, :b, %{c: 1111})\n    \n... to make a *new* map `%{b: %{c: 1111}}`. And, retreating further up the pipeline, we\nget this:\n\n    Map.put(%{a: %{b: %{c: 1}}}, :a, %{b: %{c: 1111}})\n    \nEvery step of retreat \"back up to\" the original container allocates a\nnew map. That's just the way it is in a language without\nmutability. (Such languages can make optimizations to share structure\nbetween original and updated versions of structures, so long as no\nuser code can tell. I don't know if the Erlang virtual machine's\noptimizations for maps would help with this example.)\n\nWith that background, here's the definition of `V2.key`:\n\n    def key(key) do\n      fn container, descender ->\n        updated =\n          Map.get(container, key)\n          |> descender.()\n\n        Map.put(container, key, updated)\n      end\n    end\n\nIn the leaf (`%{c: 1}`) case, that code has this effect:\n\n        updated =\n          Map.get(%{c: 1}, :c)\n          |> (&1 * 1111).()\n        Map.put(%{c: 1}, :c, updated)\n    \nIn the next level up, the code should look like this:\n\n        updated =\n          Map.get(%{b: {c: 1}}, :b)\n          |> leaf_descender.()\n        Map.put(%{b: %{c: 1}}, :b, updated)\n    \nHowever, there's a `V2.seq` in between the `key(:c)` leaf lens and the\npreceding `key(:b)` lens. What must that look like?\n\nThe `V1` version of `seq` gets wrapped values and has to unwrap\nthem. But this `V2` version gets a sub-container that's had a\nrepacement done. It doesn't have to do anything but return that value\nto the previous lens, which will put it into the enclosing container.\nSo that's easy:\n\n    # `get_all` version                                     # `update` version\n    def seq(outer_lens, inner_lens) do                      def seq(outer_lens, inner_lens) do\n      fn outer_container, inner_descender ->                  fn outer_container, inner_descender ->\n        outer_descender =                                       outer_descender =\n          fn inner_container ->                                   fn inner_container ->\n            inner_lens.(inner_container, inner_descender)           inner_lens.(inner_container, inner_descender)\n          end                                                     end\n                                                               \n        gotten =                                                updated =\n        ^^^^^^                                                  ^^^^^^^\n          outer_lens.(outer_container, outer_descender)           outer_lens.(outer_container, outer_descender)\n                                                             \n        Enum.concat(gotten)                                     updated\n        ^^^^^^^^^^^^^^^^^^^                                     ^^^^^^^\n      end                                                     end\n    end                                                     end\n\n\nWhen it comes to ordinary lenses, the changes are equally trivial. Here's `at`:\n\n    # `get_all` version                # `update` version\n    def at(index) do                   def at(index) do\n      fn container, descender ->        fn container, descender ->\n        gotten =                          updated =\n          Enum.at(container, index)         Enum.at(container, index)\n          |> descender.()                   |> descender.()\n        [gotten]                          List.replace_at(container, index, updated)\n        ^^^^^^^^                          ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n      end                               end\n    end                               end\n\n`all` doesn't have to change at all:\n\n    def all do                          def all do\n      fn container, descender ->          fn container, descender ->\n        for item <- container,              for item <- container,\n            do: descender.(item)                do: descender.(item)\n      end                                 end\n    end                                 end\n\nNext is to combine the V1 and V2 lenses into a template that works for both getting and updating.","ref":"implementation03-update.html"},{"type":"extras","title":"Version 3: get_and_update","doc":"# Version 3: `get_and_update`\n\nThe code and tests for this version can be found in\n[`implementation_v3_get_and_update_test.exs`](../test/mostly_words/tutorial/implementation_v3_get_and_update_test.exs).\n\n\n`Lens2.Deeply.get_and_update/3` is analogous to Elixir'sbuilt-in\n`get_and_update_in/3`. As with `get_and_update_in/3`, it takes a function\nthat returns both an original value and its updated version:\n\n    iex> tuple_returner = fn value -> {value, inspect(value)} end\n    iex> Deeply.get_and_update(%{a: 1}, Lens.key(:a), tuple_returner)\n    {[1], %{a: \"1\"}}\n\nTo implement this:\n\n1. The descender function will, as before, take as its argument some\n   container. But it will return a `{gotten, updated}` tuple.\n   \n2. A lens function will also return such a tuple to its caller.\n\n3. `Derply.get_and_update` can just return the tuple:\n\n        def get_and_update(container, lens, tuple_returner) do\n          lens.(container, tuple_returner)\n        end\n\nHere is a new version of `at`. It gets a value, sends it to the\n`descender`, and processes the two returned elements as in version 1\nand version 2 (respectively):\n\n    def at(index) do\n      fn container, descender ->\n        {gotten, updated} =\n         ^^^^^^  ^^^^^^^\n          Enum.at(container, index)\n          |> descender.()\n\n        {[gotten], List.replace_at(container, index, updated)}\n         ^^^^^^^^  ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n      end\n    end\n    \nNot a very big change. The same can be done for `seq`:\n\n    def seq(outer_lens, inner_lens) do\n      fn outer_container, inner_descender ->\n        outer_descender =\n          fn inner_container ->\n            inner_lens.(inner_container, inner_descender)\n          end\n\n        {gotten, updated} =\n         ^^^^^^  ^^^^^^^\n          outer_lens.(outer_container, outer_descender)\n\n        {Enum.concat(gotten), updated}\n         ^^^^^^^^^^^^^^^^^^^  ^^^^^^^\n      end\n    end","ref":"implementation04-get_and_update.html"},{"type":"extras","title":"The other `Derply` operations - Version 3: get_and_update","doc":"`Derply.update/3` uses a function that returns an updated container\n(not a `{gotten, updated}` tuple). By converting that update function\ninto a \"tuple returner\", it can just use `Derply.get_and_update` and\nignore the `gotten` value:\n\n    def update(container, lens, update_fn) do\n      tuple_returner = & {&1, update_fn.(&1)}\n                       ^^^^^^^^^^^^^^^^^^^^^^\n      {_, updated} = get_and_update(container, lens, tuple_returner)\n                                                     ^^^^^^^^^^^^^^\n      updated\n    end\n\n`get_all` also calls `get_and_update`; it just returns the `gotten`\ntuple element instead of `updated`. But there's a problem: What\ntuple-returning function does it call?\n\n    def get_all(container, lens) do\n      tuple_returner = & {&1, ?????????}\n                              ^^^^^^^^^\n      {gotten, _} = get_and_update(container, lens, tuple_returner)\n      gotten\n    end\n\nThere's a sense in which it doesn't matter, because whatever update\nhappens is thrown away. So it uses the most innocuous function\npossible: the identity function:\n\n      tuple_returner = & {&1, &1}\n\n## [Record scratch noise](pics/record-scratch.mp3)\n\nBut wait. The `at` lens's return value is constructed like this:\n\n      {gotten, updated} = ...\n\n      {[gotten],\n       List.replace_at(container, index, updated)","ref":"implementation04-get_and_update.html#the-other-derply-operations"},{"type":"extras","title":"‽‽‽‽‽ - Version 3: get_and_update","doc":"}\n\nIsn't `List.replace_at/3` going to construct a *new* list, one that's\nidentical to the old list?  And in a pipeline of lenses, won't there\nbe a whole series of lenses doing that – going to a lot of work to\nconstruct a container that's `==` to the original container,\nallocating memory like mad? Only to throw away the newly-constructed\ncontainer?\n\nWell, yes. In the case of `at`, that's exactly what\nhappens. Fortunately, composite structures like `Map`, structs, and\n`MapSet` are clever enough not to allocate a new structure if they see\nyou're trying to `put` the same value that's already there. Instead,\nthey just return the original structure.\n\nStill, for a large list (and other not-so-clever datetypes), there's\npotentially a lot of wasted work. That means that `Deeply.get` might\nget progressively slower than `get_in` as containers get bigger. \n\nI did some **extremely** crude benchmarking. For maps and structs, are\nabout twice as slow as `Access` functions, and the relationship seems\nconstant(ish). (I used the real `Lens` code, not `V3`.) For lists,\nlenses get slower and slower as the lists get longer, but there's not\nan alarming \"knee\" in a graph of 4,000,000 repetitions of probes into\nlists of sizes up to 10,000 elements:\n\n\n![alt text coming](pics/get_in_vs_get_all.png)\n\nThe ancient advice applies: let profiling tell you where your problems are.\n\n(`Access` avoids the problem of pointless updates by having two paths\nthrough the equivalent of lens functions: one that's purely a \"get\"\nand one that's a \"get and update\". That means it's more work to add a\nnew data structure to `Access` than it is to write a lens for it, but you're more likely to need to replace a lens pipeline with a hand-coded get or update function.)\n\nSpeaking of `Access`...","ref":"implementation04-get_and_update.html#"},{"type":"extras","title":"Version 4: compatibility with Access","doc":"# Version 4: compatibility with `Access`\n\nThe code and tests for this version can be found in\n[`implementation_v4_access_test.exs`](../test/mostly_words/tutorial/implementation_v4_access_test.exs).\n\n\nWe want lenses to be compatible with `get_in/2` and friends:\n\n\n    iex> tuple_returner = & {&1, inspect(&1)}\n    iex> lens = Lens.at(1)\n    iex> get_and_update_in([0, 1, 2], [lens], tuple_returner)\n                                      ^^^^^^\n    {[1], [0, \"1\", 2]}\n    \n\n... including as only part of a list argument:\n\n\n    iex> container = [0, %{a: 1}, 2]\n    iex> get_and_update_in(container, [lens, :a], tuple_returner)\n                                      ^^^^^^^^^^\n    {[1], [0, %{a: \"1\"}, 2]}\n\nIn fact, let's just define our `Derply` functions in terms of the Elixir built-ins:\n\n\n    def get_and_update(container, lens, tuple_returner) do\n      Kernel.get_and_update_in(container, [lens], tuple_returner)\n    end\n\n    def update(container, lens, update_fn) do\n      Kernel.update_in(container, [lens], update_fn)\n    end\n\n    def get_all(container, lens) do\n      Kernel.get_in(container, [lens])\n    end\n\nActually, we don't need to define `Derply` functions at all, as these\nare the actual definitions used in `Lens2.Deeply`.","ref":"implementation05-access.html"},{"type":"extras","title":"The behaviour - Version 4: compatibility with Access","doc":"A function suitable for `Access` is must have the following interface:\n\n          fn\n            :get, container, continuation ->\n               ...\n\n            :get_and_update, container, tuple_returner ->\n               ...\n          end\n\nThe `:get` and `:get_and_update` arguments are required to distinguish\nthe two branches because the tyhpes of the second and third arguments\nare the same in both functions. The `container` can be `any` type, and\nboth `continuation` and `tuple_returner` are arity-one\nfunctions. Let's look at the body of the `:get` case first:\n\n            :get, container, continuation ->\n              {gotten, _} = lens.(container, &{&1, &1})\n              continuation.(gotten)\n\nExcept for the the use of the `continuation` argument, this does the\nsame thing as the V3 version of `get_all`. In particular it uses the\ntuple-returner `&{&1, &1}` to produce the throw-away version of the\n`updated` container. (So the wasteful multi-level allocation happens with\n`get_in/2`.)\n\nThe continuation represents what comes after this function in a call to `get_in`. So, in this:\n\n\n    get_in(..., [Lens.at(0), :a, :b])\n\n... the continuation is a function representing the descent through\nthe keys `:a` and `:b`. Because `Deeply.get_all` uses a singleton\nlist:\n\n    def get_all(container, lens) do\n      Kernel.get_in(container, [lens])\n                               ^^^^^^\n    end\n\n... there's nothing more to do, so the `continuation` will be our old\nfriend `& &1`. That is, when it comes to lenses, we don't need to\nworry about it.\n\nThe `:get_and_update` clause looks just like the code for version 3's `Derply.get_and_update`:\n\n            :get_and_update, container, tuple_returner ->\n              lens.(container, tuple_returner)\n\n\nThe code for the actual lenses is the same as in version 3, except\nthat it should use the macro `Lens2.Makers.def_raw_maker/2` instead of\n`def`. `def_raw_maker` wraps its body in a `:get`/`:get_and_update`\nfunction, and also arranges to create the lens maker with an\nadditional argument – the one used in a pipeline. So, this use of `def_raw_maker`:\n\n    def_raw_maker at(index) do\n      fn container, descender -> ... end\n    end\n\n... will expand out to:\n\n    def at(index) do \n      lens = fn container, descender -> ... end         # <<<<  \n           {gotten, _} = lens.(container, &{&1, &1})\n           continuation.(gotten)\n\n         :get_and_update, container, tuple_returner ->\n           lens.(container, tuple_returner)\n       end\n    end\n    \n    def at(previously, index) do \n      Lens.seq(previous, at(index))\n    end","ref":"implementation05-access.html#the-behaviour"},{"type":"extras","title":"Example: Lens.into","doc":"# Example: `Lens.into`","ref":"implementation06-into.html"},{"type":"extras","title":"Example: Lens.const","doc":"# Example: `Lens.const`","ref":"implementation07-const.html"},{"type":"extras","title":"Some exemplary implementations","doc":"# Some exemplary implementations","ref":"implementation08-some-makers.html"},{"type":"extras","title":"DRAFT: Popping","doc":"# DRAFT: Popping\n\n`Access` supports \"popping\" elements from containers. There are two variants.\n\n\n1. The first variant allows the tuple-returner passed to\n   `get_and_update_in` to return a `:pop` value instead of a tuple. That informs\n   the calling code to remove an element. Here's a rather contrived example. It pops\n   a key if its value is negative, stringifies it otherwise:\n\n        iex> pop_negative = \n        ...>  & if &1 < 0, do: :pop, else: {&1, inspect(&1)}\n        iex> container = [%{a: 1}, %{a: -1}]\n        iex> get_and_update_in(container, [Access.all, :a], pop_negative)\n        {[1, -1], [%{a: \"1\"}, %{}]}\n\n2. The second variant pops particular keys, regardless of their values:\n\n        iex> container = [ %{a: %{aa: 1, bb: 2}},\n                           %{a: %{aa: 11, bb: 22}}]\n        iex> pop_in(container, [Access.all, :a, :aa])\n        {[1, 11], [%{a: %{bb: 2}}, %{a: %{bb: 22}}]}","ref":"draft_popping.html"},{"type":"extras","title":"The problem with `get_and_update` - DRAFT: Popping","doc":"`Access` works on a few data types and requires them each to implement\n`get_and_update`. It's that data-structure-specific function that\nhandles `:pop` return values. Here, for example is `Map.get_and_update/3`\n\n       def get_and_update(map, key, tuple_returner)\n         current = get(map, key)\n\n         case tuple_returner.(current) do\n           {gotten, updated} ->\n             {gotten, put(map, key, updated)}\n\n           :pop ->\n             {current, delete(map, key)}\n         end\n       end\n       \nWe could not put a similar `:pop` clause in the function that\n`def_raw_maker` wraps around a lens function because it's too late. The\nwrapper takes control *after* the lens (and descender) have modified\nits container.  So a `:pop` case for a map would be working with an\n`updated` value like `%{a: :pop}`, not a simple `:pop`-or-tuple return\nvalue. Whereas `Map.get_and_update/3` has to work with exactly one\ndatatype, the generic wrapper would have to work with\n*everything*. (And, given lenses like\n`Lens2.Lenses.Combine.repeatedly/1`, it couldn't even just scan the\ntop level for `:pop`.)\n\nSo supporting this kind of popping would mean we'd have to change quite a number\nof lenses: \n\n    def_raw_maker key(key) do\n      fn container, descender ->\n        {gotten, updated} = descender.(Map.get(container, key))\n        {[gotten], Map.put(container, key, updated)}\n      end\n    end\n    \n... to this:\n\n    def_raw_maker key(key) do\n      fn container, descender ->\n\n        current = Map.get(container, key)\n        case descender.(current) do\n          :pop ->\n            {[current], Map.delete(container, key)}\n          {gotten, updated} ->\n            {[gotten], Map.put(container, key, updated)}\n        end\n      end\n    end\n\nThe same would have to be done for `key?/1`, `key!/1`, `at/1`. So it's\nnot surprising the original author of Lens 1 left it out.\n\n... though, now that I think about it, it wouldn't be *so* hard.","ref":"draft_popping.html#the-problem-with-get_and_update"},{"type":"extras","title":"The problem with pop_in - DRAFT: Popping","doc":"You can also directly instruct `Access`-compatible types to pop an element at a named key:\n\n\n    iex> container = %{a: [0, 1, 2]}\n    iex> pop_in(container, [:a, Access.at(1)])\n    {1, %{a: [0, 2]}}\n\nIn this case, the accessor list contains a function (`Access.at(1)`),\nso it's handled as in the `get_and_access_in`. Effectively, it's turned into:\n\n    iex> always_pop = fn _ -> :pop end\n    iex> get_and_update_in(container, [:a, Access.at(0)], always_pop)\n    {0, %{a: [1, 2]}}\n    \nIn the more common case, where all the access list elements are keys,\nit's handled by the `Access`-compatible datatype's `pop(container,\nkey)` function, applied to the final container and key. So, in lens terminology,\nthe final descender is (for a `Map`):\n\n\n    descender = fn Map.pop(%{a: \"pop\", b: \"no\"}, :a) end\n    \nThe `Access` code can do that because the access list is a *list*, so\nit can inspect the last element as it recursively descends the\ncontainer. The actual code pattern matches on a single-element\nlist. See the fifth and sixth lines below (from the `Access.pop/2` source):\n\n\n    defp pop_in_data(data, [fun])        when is_function(fun),\n      ...\n    defp pop_in_data(data, [fun | tail]) when is_function(fun),\n      fun.(:get_and_update, data, fn _ -> :pop end)\n    defp pop_in_data(data, [key]),                  # <<<<\n      do: Access.pop(data, key)                     # <<<<\n    defp pop_in_data(data, [key | tail]),\n      ...\n    \nLens code operates on functions, not lists of keys and functions, so\nit can't know when it's descended to the final element. Again, any\nimplementation of `Deeply.pop_in` would devolve to changing lenses to\nhandle a `:pop` result returned from a descender.","ref":"draft_popping.html#the-problem-with-pop_in"},{"type":"extras","title":"DRAFT: Why Lens.into invites bugs","doc":"# DRAFT: Why `Lens.into` invites bugs\n\nHere I'll describe a very common mistake using\n`Lens2.Lenses.Enum.into/2`. This is sort of a digression, but it\nreinforces some ideas from the previous two pages.\n\n`Lens2.Lenses.Enum.all/0` turns a pointer to an `Enumerable` into a set of\npointers to all its elements. It's most often used with lists, but it\ndoesn't have to be.\n\n     iex> use Lens2\n     iex> Deeply.get_all(1..5, Lens.all)\n     [1, 2, 3, 4, 5]\n\nSuppose we want to increment each of the numbers. That doesn't\nactually make sense for a range, but let's see what happens:\n\n    iex> Deeply.update(1..5, Lens.all, & &1+1)\n    [2, 3, 4, 5, 6]\n\nOr just overwrite all the values:\n\n    iex> Deeply.put(1..5, Lens.all, 1111)\n    [1111, 1111, 1111, 1111, 1111]\n\n\nFor any update operation, `Lens.all/0` produces a list. Suppose we\ninstead want a `MapSet`. We could do that with `Enum.into/2`:\n\n    iex> Deeply.put(1..5, Lens.all, 1111) |> Enum.into(MapSet.new)\n    MapSet.new([1111])\n\n(Notice that collapsed all the `11111` values into one, because\nMapSets don't allow duplicates. Maybe that's why we wanted a `MapSet`.)\n\nThere is, however, a lens that is the equivalent of `Enum.into/2`:\n`Lens2.Lenses.Enum.into/2`:\n\n    iex> Deeply.put(1..5, Lens.all |> Lens.into(MapSet.new), 11111)\n    MapSet.new([11111])\n\nLooks good. Let's even put it in a module a a predefined lens maker:\n\n    defmodule MyLenses do\n      defmaker as_mapset,\n        do: Lens.all |> Lens.into(MapSet.new)\n    end\n\nNow it happens that we have a structure containing a range, and we\nwant to increment the range values into a mapset. Seems easy:\n\n\n    iex> lens = Lens.key(:a) |> MyLenses.as_mapset\n    iex> Deeply.update(%{a: 1..5}, lens, & &1+1)\n    %{a: MapSet.new([2, 3, 4, 5, 6])}\n    \nThat looks good. Time passes, and you come across a similar situation. This time you decide a named lens maker is overkill. You'll just pipe the lenses together at the point of use:\n\n    iex> lens = Lens.key(:a) |> Lens.all |> Lens.into(MapSet.new)\n    iex> Deeply.update(%{a: 1..5}, lens, & &1+1)\n    MapSet.new([a: [2, 3, 4, 5, 6]])\n    \nLook closely at that: it's not a map containing a mapset. It's a\nmapset containing a keyword list.\n\nTwo questions: \n\n1. What went wrong with this?\n\n        Lens.key(:a) |> Lens.all |> Lens.into(MapSet.new)\n\n2. And why did *this* work...?\n\n        lens = Lens.key(:a) |> MyLenses.as_mapset\n    \n   ...given that `as_mapset` is just:\n\n        Lens.all |> Lens.into(MapSet.new)","ref":"draft_into.html"},{"type":"extras","title":"Why the pipeline fails - DRAFT: Why Lens.into invites bugs","doc":"What if we manually expand out the pipeline into a nested pair of `Lens.seq`?\n\n    Lens.seq(Lens.key(:a), Lens.seq(Lens.all, Lens.into(MapSet.new)))\n    ** (UndefinedFunctionError) function Lens2.Lenses.into/1 is undefined or private. Did you mean:\n\n      * into/2\n      * into/3\n\n`Lens.all |>`","ref":"draft_into.html#why-the-pipeline-fails"},{"type":"extras","title":"Why the named function doesn't fail - DRAFT: Why Lens.into invites bugs","doc":"","ref":"draft_into.html#why-the-named-function-doesn-t-fail"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}